/* Copyright (c) 2014, The Linux Foundation. All rights reserved.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 and
* only version 2 as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*/

#include <linux/mfd/wcd9xxx/wcd9330_registers.h>
#include "wcd9330.h"

const u8 tomtom_reg_readable[TOMTOM_CACHE_SIZE] = {
	[TOMTOM_A_CHIP_CTL] = 1,
	[TOMTOM_A_CHIP_STATUS] = 1,
	[TOMTOM_A_CHIP_ID_BYTE_0] = 1,
	[TOMTOM_A_CHIP_ID_BYTE_1] = 1,
	[TOMTOM_A_CHIP_ID_BYTE_2] = 1,
	[TOMTOM_A_CHIP_ID_BYTE_3] = 1,
	[TOMTOM_A_CHIP_I2C_SLAVE_ID] = 1,
	[TOMTOM_A_SLAVE_ID_1] = 1,
	[TOMTOM_A_SLAVE_ID_2] = 1,
	[TOMTOM_A_SLAVE_ID_3] = 1,
	[TOMTOM_A_PIN_CTL_OE0] = 1,
	[TOMTOM_A_PIN_CTL_OE1] = 1,
	[TOMTOM_A_PIN_CTL_OE2] = 1,
	[TOMTOM_A_PIN_CTL_DATA0] = 1,
	[TOMTOM_A_PIN_CTL_DATA1] = 1,
	[TOMTOM_A_PIN_CTL_DATA2] = 1,
	[TOMTOM_A_HDRIVE_GENERIC] = 1,
	[TOMTOM_A_HDRIVE_OVERRIDE] = 1,
	[TOMTOM_A_ANA_CSR_WAIT_STATE] = 1,
	[TOMTOM_A_PROCESS_MONITOR_CTL0] = 1,
	[TOMTOM_A_PROCESS_MONITOR_CTL1] = 1,
	[TOMTOM_A_PROCESS_MONITOR_CTL2] = 1,
	[TOMTOM_A_PROCESS_MONITOR_CTL3] = 1,
	[TOMTOM_A_QFUSE_CTL] = 1,
	[TOMTOM_A_QFUSE_STATUS] = 1,
	[TOMTOM_A_QFUSE_DATA_OUT0] = 1,
	[TOMTOM_A_QFUSE_DATA_OUT1] = 1,
	[TOMTOM_A_QFUSE_DATA_OUT2] = 1,
	[TOMTOM_A_QFUSE_DATA_OUT3] = 1,
	[TOMTOM_A_QFUSE_DATA_OUT4] = 1,
	[TOMTOM_A_QFUSE_DATA_OUT5] = 1,
	[TOMTOM_A_QFUSE_DATA_OUT6] = 1,
	[TOMTOM_A_QFUSE_DATA_OUT7] = 1,
	[TOMTOM_A_CDC_CTL] = 1,
	[TOMTOM_A_LEAKAGE_CTL] = 1,
	[TOMTOM_A_SVASS_MEM_PTR0] = 1,
	[TOMTOM_A_SVASS_MEM_PTR1] = 1,
	[TOMTOM_A_SVASS_MEM_PTR2] = 1,
	[TOMTOM_A_SVASS_MEM_CTL] = 1,
	[TOMTOM_A_SVASS_MEM_BANK] = 1,
	[TOMTOM_A_DMIC_B1_CTL] = 1,
	[TOMTOM_A_DMIC_B2_CTL] = 1,
	[TOMTOM_A_SVASS_CLKRST_CTL] = 1,
	[TOMTOM_A_SVASS_CPAR_CFG] = 1,
	[TOMTOM_A_SVASS_BUF_RDY_INT_PERIOD] = 1,
	[TOMTOM_A_SVASS_CPAR_WDOG_CFG] = 1,
	[TOMTOM_A_SVASS_CFG] = 1,
	[TOMTOM_A_SVASS_SPE_CFG] = 1,
	[TOMTOM_A_SVASS_STATUS] = 1,
	[TOMTOM_A_SVASS_INT_MASK] = 1,
	[TOMTOM_A_SVASS_INT_STATUS] = 1,
	[TOMTOM_A_SVASS_INT_CLR] = 0,
	[TOMTOM_A_SVASS_DEBUG] = 1,
	[TOMTOM_A_SVASS_SPE_BKUP_INT] = 0,
	[TOMTOM_A_SVASS_MEM_ACC] = 1,
	[TOMTOM_A_MEM_LEAKAGE_CTL] = 1,
	[TOMTOM_A_SVASS_SPE_INBOX_TRG] = 0,
	[TOMTOM_A_SVASS_SPE_INBOX_0] = 0,
	[TOMTOM_A_SVASS_SPE_INBOX_1] = 0,
	[TOMTOM_A_SVASS_SPE_INBOX_2] = 0,
	[TOMTOM_A_SVASS_SPE_INBOX_3] = 0,
	[TOMTOM_A_SVASS_SPE_INBOX_4] = 0,
	[TOMTOM_A_SVASS_SPE_INBOX_5] = 0,
	[TOMTOM_A_SVASS_SPE_INBOX_6] = 0,
	[TOMTOM_A_SVASS_SPE_INBOX_7] = 0,
	[TOMTOM_A_SVASS_SPE_INBOX_8] = 0,
	[TOMTOM_A_SVASS_SPE_INBOX_9] = 0,
	[TOMTOM_A_SVASS_SPE_INBOX_10] = 0,
	[TOMTOM_A_SVASS_SPE_INBOX_11] = 0,
	[TOMTOM_A_SVASS_SPE_OUTBOX_0] = 1,
	[TOMTOM_A_SVASS_SPE_OUTBOX_1] = 1,
	[TOMTOM_A_SVASS_SPE_OUTBOX_2] = 1,
	[TOMTOM_A_SVASS_SPE_OUTBOX_3] = 1,
	[TOMTOM_A_SVASS_SPE_OUTBOX_4] = 1,
	[TOMTOM_A_SVASS_SPE_OUTBOX_5] = 1,
	[TOMTOM_A_SVASS_SPE_OUTBOX_6] = 1,
	[TOMTOM_A_SVASS_SPE_OUTBOX_7] = 1,
	[TOMTOM_A_SVASS_SPE_OUTBOX_8] = 1,
	[TOMTOM_A_SVASS_SPE_OUTBOX_9] = 1,
	[TOMTOM_A_SVASS_SPE_OUTBOX_10] = 1,
	[TOMTOM_A_SVASS_SPE_OUTBOX_11] = 1,
	[TOMTOM_A_INTR_MODE] = 1,
	[TOMTOM_A_INTR1_MASK0] = 1,
	[TOMTOM_A_INTR1_MASK1] = 1,
	[TOMTOM_A_INTR1_MASK2] = 1,
	[TOMTOM_A_INTR1_MASK3] = 1,
	[TOMTOM_A_INTR1_STATUS0] = 1,
	[TOMTOM_A_INTR1_STATUS1] = 1,
	[TOMTOM_A_INTR1_STATUS2] = 1,
	[TOMTOM_A_INTR1_STATUS3] = 1,
	[TOMTOM_A_INTR1_CLEAR0] = 0,
	[TOMTOM_A_INTR1_CLEAR1] = 0,
	[TOMTOM_A_INTR1_CLEAR2] = 0,
	[TOMTOM_A_INTR1_CLEAR3] = 0,
	[TOMTOM_A_INTR1_LEVEL0] = 1,
	[TOMTOM_A_INTR1_LEVEL1] = 1,
	[TOMTOM_A_INTR1_LEVEL2] = 1,
	[TOMTOM_A_INTR1_LEVEL3] = 1,
	[TOMTOM_A_INTR1_TEST0] = 1,
	[TOMTOM_A_INTR1_TEST1] = 1,
	[TOMTOM_A_INTR1_TEST2] = 1,
	[TOMTOM_A_INTR1_TEST3] = 1,
	[TOMTOM_A_INTR1_SET0] = 1,
	[TOMTOM_A_INTR1_SET1] = 1,
	[TOMTOM_A_INTR1_SET2] = 1,
	[TOMTOM_A_INTR1_SET3] = 1,
	[TOMTOM_A_INTR2_MASK0] = 1,
	[TOMTOM_A_INTR2_STATUS0] = 1,
	[TOMTOM_A_INTR2_CLEAR0] = 0,
	[TOMTOM_A_INTR2_LEVEL0] = 1,
	[TOMTOM_A_INTR2_TEST0] = 1,
	[TOMTOM_A_INTR2_SET0] = 1,
	[TOMTOM_A_CDC_TX_I2S_SCK_MODE] = 1,
	[TOMTOM_A_CDC_TX_I2S_WS_MODE] = 1,
	[TOMTOM_A_CDC_DMIC_DATA0_MODE] = 1,
	[TOMTOM_A_CDC_DMIC_CLK0_MODE] = 1,
	[TOMTOM_A_CDC_DMIC_DATA1_MODE] = 1,
	[TOMTOM_A_CDC_DMIC_CLK1_MODE] = 1,
	[TOMTOM_A_CDC_RX_I2S_SCK_MODE] = 1,
	[TOMTOM_A_CDC_RX_I2S_WS_MODE] = 1,
	[TOMTOM_A_CDC_DMIC_DATA2_MODE] = 1,
	[TOMTOM_A_CDC_DMIC_CLK2_MODE] = 1,
	[TOMTOM_A_CDC_INTR1_MODE] = 1,
	[TOMTOM_A_CDC_SB_NRZ_SEL_MODE] = 1,
	[TOMTOM_A_CDC_INTR2_MODE] = 1,
	[TOMTOM_A_CDC_RF_PA_ON_MODE] = 1,
	[TOMTOM_A_CDC_BOOST_MODE] = 1,
	[TOMTOM_A_CDC_JTCK_MODE] = 1,
	[TOMTOM_A_CDC_JTDI_MODE] = 1,
	[TOMTOM_A_CDC_JTMS_MODE] = 1,
	[TOMTOM_A_CDC_JTDO_MODE] = 1,
	[TOMTOM_A_CDC_JTRST_MODE] = 1,
	[TOMTOM_A_CDC_BIST_MODE_MODE] = 1,
	[TOMTOM_A_CDC_MAD_MAIN_CTL_1] = 1,
	[TOMTOM_A_CDC_MAD_MAIN_CTL_2] = 1,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_1] = 1,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_2] = 1,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_3] = 1,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_4] = 1,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_5] = 1,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_6] = 1,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_7] = 1,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_8] = 1,
	[TOMTOM_A_CDC_MAD_AUDIO_IIR_CTL_PTR] = 1,
	[TOMTOM_A_CDC_MAD_AUDIO_IIR_CTL_VAL] = 1,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_1] = 1,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_2] = 1,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_3] = 1,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_4] = 1,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_5] = 1,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_6] = 1,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_7] = 1,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_1] = 1,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_2] = 1,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_3] = 1,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_4] = 1,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_5] = 1,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_6] = 1,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_7] = 1,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_8] = 1,
	[TOMTOM_A_CDC_MAD_BEACON_IIR_CTL_PTR] = 1,
	[TOMTOM_A_CDC_MAD_BEACON_IIR_CTL_VAL] = 1,
	[TOMTOM_A_CDC_MAD_INP_SEL] = 1,
	[TOMTOM_A_BIAS_REF_CTL] = 1,
	[TOMTOM_A_BIAS_CENTRAL_BG_CTL] = 1,
	[TOMTOM_A_BIAS_PRECHRG_CTL] = 1,
	[TOMTOM_A_BIAS_CURR_CTL_1] = 1,
	[TOMTOM_A_BIAS_CURR_CTL_2] = 1,
	[TOMTOM_A_BIAS_OSC_BG_CTL] = 1,
	[TOMTOM_A_CLK_BUFF_EN1] = 1,
	[TOMTOM_A_CLK_BUFF_EN2] = 1,
	[TOMTOM_A_LDO_L_MODE_1] = 1,
	[TOMTOM_A_LDO_L_MODE_2] = 1,
	[TOMTOM_A_LDO_L_CTRL_1] = 1,
	[TOMTOM_A_LDO_L_CTRL_2] = 1,
	[TOMTOM_A_LDO_L_CTRL_3] = 1,
	[TOMTOM_A_LDO_L_CTRL_4] = 1,
	[TOMTOM_A_LDO_H_MODE_1] = 1,
	[TOMTOM_A_LDO_H_MODE_2] = 1,
	[TOMTOM_A_LDO_H_LOOP_CTL] = 1,
	[TOMTOM_A_LDO_H_COMP_1] = 1,
	[TOMTOM_A_LDO_H_COMP_2] = 1,
	[TOMTOM_A_LDO_H_BIAS_1] = 1,
	[TOMTOM_A_LDO_H_BIAS_2] = 1,
	[TOMTOM_A_LDO_H_BIAS_3] = 1,
	[TOMTOM_A_VBAT_CLK] = 1,
	[TOMTOM_A_VBAT_LOOP] = 1,
	[TOMTOM_A_VBAT_REF] = 1,
	[TOMTOM_A_VBAT_ADC_TEST] = 1,
	[TOMTOM_A_VBAT_FE] = 1,
	[TOMTOM_A_VBAT_BIAS_1] = 1,
	[TOMTOM_A_VBAT_BIAS_2] = 1,
	[TOMTOM_A_VBAT_ADC_DATA_MSB] = 1,
	[TOMTOM_A_VBAT_ADC_DATA_LSB] = 1,
	[TOMTOM_A_FLL_NREF] = 1,
	[TOMTOM_A_FLL_KDCO_TUNE] = 1,
	[TOMTOM_A_FLL_LOCK_THRESH] = 1,
	[TOMTOM_A_FLL_LOCK_DET_COUNT] = 1,
	[TOMTOM_A_FLL_DAC_THRESHOLD] = 1,
	[TOMTOM_A_FLL_TEST_DCO_FREERUN] = 1,
	[TOMTOM_A_FLL_TEST_ENABLE] = 1,
	[TOMTOM_A_MICB_CFILT_1_CTL] = 1,
	[TOMTOM_A_MICB_CFILT_1_VAL] = 1,
	[TOMTOM_A_MICB_CFILT_1_PRECHRG] = 1,
	[TOMTOM_A_MICB_1_CTL] = 1,
	[TOMTOM_A_MICB_1_INT_RBIAS] = 1,
	[TOMTOM_A_MICB_1_MBHC] = 1,
	[TOMTOM_A_MICB_CFILT_2_CTL] = 1,
	[TOMTOM_A_MICB_CFILT_2_VAL] = 1,
	[TOMTOM_A_MICB_CFILT_2_PRECHRG] = 1,
	[TOMTOM_A_MICB_2_CTL] = 1,
	[TOMTOM_A_MICB_2_INT_RBIAS] = 1,
	[TOMTOM_A_MICB_2_MBHC] = 1,
	[TOMTOM_A_MICB_CFILT_3_CTL] = 1,
	[TOMTOM_A_MICB_CFILT_3_VAL] = 1,
	[TOMTOM_A_MICB_CFILT_3_PRECHRG] = 1,
	[TOMTOM_A_MICB_3_CTL] = 1,
	[TOMTOM_A_MICB_3_INT_RBIAS] = 1,
	[TOMTOM_A_MICB_3_MBHC] = 1,
	[TOMTOM_A_MICB_4_CTL] = 1,
	[TOMTOM_A_MICB_4_INT_RBIAS] = 1,
	[TOMTOM_A_MICB_4_MBHC] = 1,
	[TOMTOM_A_SPKR_DRV2_EN] = 1,
	[TOMTOM_A_SPKR_DRV2_GAIN] = 1,
	[TOMTOM_A_SPKR_DRV2_DAC_CTL] = 1,
	[TOMTOM_A_SPKR_DRV2_OCP_CTL] = 1,
	[TOMTOM_A_SPKR_DRV2_CLIP_DET] = 1,
	[TOMTOM_A_SPKR_DRV2_DBG_DAC] = 1,
	[TOMTOM_A_SPKR_DRV2_DBG_PA] = 1,
	[TOMTOM_A_SPKR_DRV2_DBG_PWRSTG] = 1,
	[TOMTOM_A_SPKR_DRV2_BIAS_LDO] = 1,
	[TOMTOM_A_SPKR_DRV2_BIAS_INT] = 1,
	[TOMTOM_A_SPKR_DRV2_BIAS_PA] = 1,
	[TOMTOM_A_SPKR_DRV2_STATUS_OCP] = 1,
	[TOMTOM_A_SPKR_DRV2_STATUS_PA] = 1,
	[TOMTOM_A_MBHC_INSERT_DETECT] = 1,
	[TOMTOM_A_MBHC_INSERT_DET_STATUS] = 1,
	[TOMTOM_A_TX_COM_BIAS] = 1,
	[TOMTOM_A_MBHC_INSERT_DETECT2] = 1,
	[TOMTOM_A_MBHC_SCALING_MUX_1] = 1,
	[TOMTOM_A_MBHC_SCALING_MUX_2] = 1,
	[TOMTOM_A_MAD_ANA_CTRL] = 1,
	[TOMTOM_A_TX_SUP_SWITCH_CTRL_1] = 1,
	[TOMTOM_A_TX_SUP_SWITCH_CTRL_2] = 1,
	[TOMTOM_A_TX_1_GAIN] = 1,
	[TOMTOM_A_TX_1_2_TEST_EN] = 1,
	[TOMTOM_A_TX_2_GAIN] = 1,
	[TOMTOM_A_TX_1_2_ADC_IB] = 1,
	[TOMTOM_A_TX_1_2_ATEST_REFCTRL] = 1,
	[TOMTOM_A_TX_1_2_TEST_CTL] = 1,
	[TOMTOM_A_TX_1_2_TEST_BLOCK_EN] = 1,
	[TOMTOM_A_TX_1_2_TXFE_CLKDIV] = 1,
	[TOMTOM_A_TX_1_2_SAR_ERR_CH1] = 1,
	[TOMTOM_A_TX_1_2_SAR_ERR_CH2] = 1,
	[TOMTOM_A_TX_3_GAIN] = 1,
	[TOMTOM_A_TX_3_4_TEST_EN] = 1,
	[TOMTOM_A_TX_4_GAIN] = 1,
	[TOMTOM_A_TX_3_4_ADC_IB] = 1,
	[TOMTOM_A_TX_3_4_ATEST_REFCTRL] = 1,
	[TOMTOM_A_TX_3_4_TEST_CTL] = 1,
	[TOMTOM_A_TX_3_4_TEST_BLOCK_EN] = 1,
	[TOMTOM_A_TX_3_4_TXFE_CKDIV] = 1,
	[TOMTOM_A_TX_3_4_SAR_ERR_CH3] = 1,
	[TOMTOM_A_TX_3_4_SAR_ERR_CH4] = 1,
	[TOMTOM_A_TX_5_GAIN] = 1,
	[TOMTOM_A_TX_5_6_TEST_EN] = 1,
	[TOMTOM_A_TX_6_GAIN] = 1,
	[TOMTOM_A_TX_5_6_ADC_IB] = 1,
	[TOMTOM_A_TX_5_6_ATEST_REFCTRL] = 1,
	[TOMTOM_A_TX_5_6_TEST_CTL] = 1,
	[TOMTOM_A_TX_5_6_TEST_BLOCK_EN] = 1,
	[TOMTOM_A_TX_5_6_TXFE_CKDIV] = 1,
	[TOMTOM_A_TX_5_6_SAR_ERR_CH5] = 1,
	[TOMTOM_A_TX_5_6_SAR_ERR_CH6] = 1,
	[TOMTOM_A_TX_7_MBHC_EN] = 1,
	[TOMTOM_A_TX_7_MBHC_ATEST_REFCTRL] = 1,
	[TOMTOM_A_TX_7_MBHC_ADC] = 1,
	[TOMTOM_A_TX_7_MBHC_TEST_CTL] = 1,
	[TOMTOM_A_TX_7_MBHC_SAR_ERR] = 1,
	[TOMTOM_A_TX_7_TXFE_CLKDIV] = 1,
	[TOMTOM_A_RCO_CTRL] = 1,
	[TOMTOM_A_RCO_CALIBRATION_CTRL1] = 1,
	[TOMTOM_A_RCO_CALIBRATION_CTRL2] = 1,
	[TOMTOM_A_RCO_CALIBRATION_CTRL3] = 1,
	[TOMTOM_A_RCO_TEST_CTRL] = 1,
	[TOMTOM_A_RCO_CALIBRATION_RESULT1] = 1,
	[TOMTOM_A_RCO_CALIBRATION_RESULT2] = 1,
	[TOMTOM_A_BUCK_MODE_1] = 1,
	[TOMTOM_A_BUCK_MODE_2] = 1,
	[TOMTOM_A_BUCK_MODE_3] = 1,
	[TOMTOM_A_BUCK_MODE_4] = 1,
	[TOMTOM_A_BUCK_MODE_5] = 1,
	[TOMTOM_A_BUCK_CTRL_VCL_1] = 1,
	[TOMTOM_A_BUCK_CTRL_VCL_2] = 1,
	[TOMTOM_A_BUCK_CTRL_VCL_3] = 1,
	[TOMTOM_A_BUCK_CTRL_CCL_1] = 1,
	[TOMTOM_A_BUCK_CTRL_CCL_2] = 1,
	[TOMTOM_A_BUCK_CTRL_CCL_3] = 1,
	[TOMTOM_A_BUCK_CTRL_CCL_4] = 1,
	[TOMTOM_A_BUCK_CTRL_PWM_DRVR_1] = 1,
	[TOMTOM_A_BUCK_CTRL_PWM_DRVR_2] = 1,
	[TOMTOM_A_BUCK_CTRL_PWM_DRVR_3] = 1,
	[TOMTOM_A_BUCK_TMUX_A_D] = 1,
	[TOMTOM_A_NCP_BUCKREF] = 1,
	[TOMTOM_A_NCP_EN] = 1,
	[TOMTOM_A_NCP_CLK] = 1,
	[TOMTOM_A_NCP_STATIC] = 1,
	[TOMTOM_A_NCP_VTH_LOW] = 1,
	[TOMTOM_A_NCP_VTH_HIGH] = 1,
	[TOMTOM_A_NCP_ATEST] = 1,
	[TOMTOM_A_NCP_DTEST] = 1,
	[TOMTOM_A_NCP_DLY1] = 1,
	[TOMTOM_A_NCP_DLY2] = 1,
	[TOMTOM_A_RX_AUX_SW_CTL] = 1,
	[TOMTOM_A_RX_PA_AUX_IN_CONN] = 1,
	[TOMTOM_A_RX_COM_TIMER_DIV] = 1,
	[TOMTOM_A_RX_COM_OCP_CTL] = 1,
	[TOMTOM_A_RX_COM_OCP_COUNT] = 1,
	[TOMTOM_A_RX_COM_DAC_CTL] = 1,
	[TOMTOM_A_RX_COM_BIAS] = 1,
	[TOMTOM_A_RX_HPH_AUTO_CHOP] = 1,
	[TOMTOM_A_RX_HPH_CHOP_CTL] = 1,
	[TOMTOM_A_RX_HPH_BIAS_PA] = 1,
	[TOMTOM_A_RX_HPH_BIAS_LDO] = 1,
	[TOMTOM_A_RX_HPH_BIAS_CNP] = 1,
	[TOMTOM_A_RX_HPH_BIAS_WG_OCP] = 1,
	[TOMTOM_A_RX_HPH_OCP_CTL] = 1,
	[TOMTOM_A_RX_HPH_CNP_EN] = 1,
	[TOMTOM_A_RX_HPH_CNP_WG_CTL] = 1,
	[TOMTOM_A_RX_HPH_CNP_WG_TIME] = 1,
	[TOMTOM_A_RX_HPH_L_GAIN] = 1,
	[TOMTOM_A_RX_HPH_L_TEST] = 1,
	[TOMTOM_A_RX_HPH_L_PA_CTL] = 1,
	[TOMTOM_A_RX_HPH_L_DAC_CTL] = 1,
	[TOMTOM_A_RX_HPH_L_ATEST] = 1,
	[TOMTOM_A_RX_HPH_L_STATUS] = 1,
	[TOMTOM_A_RX_HPH_R_GAIN] = 1,
	[TOMTOM_A_RX_HPH_R_TEST] = 1,
	[TOMTOM_A_RX_HPH_R_PA_CTL] = 1,
	[TOMTOM_A_RX_HPH_R_DAC_CTL] = 1,
	[TOMTOM_A_RX_HPH_R_ATEST] = 1,
	[TOMTOM_A_RX_HPH_R_STATUS] = 1,
	[TOMTOM_A_RX_EAR_BIAS_PA] = 1,
	[TOMTOM_A_RX_EAR_BIAS_CMBUFF] = 1,
	[TOMTOM_A_RX_EAR_EN] = 1,
	[TOMTOM_A_RX_EAR_GAIN] = 1,
	[TOMTOM_A_RX_EAR_CMBUFF] = 1,
	[TOMTOM_A_RX_EAR_ICTL] = 1,
	[TOMTOM_A_RX_EAR_CCOMP] = 1,
	[TOMTOM_A_RX_EAR_VCM] = 1,
	[TOMTOM_A_RX_EAR_CNP] = 1,
	[TOMTOM_A_RX_EAR_DAC_CTL_ATEST] = 1,
	[TOMTOM_A_RX_EAR_STATUS] = 1,
	[TOMTOM_A_RX_LINE_BIAS_PA] = 1,
	[TOMTOM_A_RX_BUCK_BIAS1] = 1,
	[TOMTOM_A_RX_BUCK_BIAS2] = 1,
	[TOMTOM_A_RX_LINE_COM] = 1,
	[TOMTOM_A_RX_LINE_CNP_EN] = 1,
	[TOMTOM_A_RX_LINE_CNP_WG_CTL] = 1,
	[TOMTOM_A_RX_LINE_CNP_WG_TIME] = 1,
	[TOMTOM_A_RX_LINE_1_GAIN] = 1,
	[TOMTOM_A_RX_LINE_1_TEST] = 1,
	[TOMTOM_A_RX_LINE_1_DAC_CTL] = 1,
	[TOMTOM_A_RX_LINE_1_STATUS] = 1,
	[TOMTOM_A_RX_LINE_2_GAIN] = 1,
	[TOMTOM_A_RX_LINE_2_TEST] = 1,
	[TOMTOM_A_RX_LINE_2_DAC_CTL] = 1,
	[TOMTOM_A_RX_LINE_2_STATUS] = 1,
	[TOMTOM_A_RX_LINE_3_GAIN] = 1,
	[TOMTOM_A_RX_LINE_3_TEST] = 1,
	[TOMTOM_A_RX_LINE_3_DAC_CTL] = 1,
	[TOMTOM_A_RX_LINE_3_STATUS] = 1,
	[TOMTOM_A_RX_LINE_4_GAIN] = 1,
	[TOMTOM_A_RX_LINE_4_TEST] = 1,
	[TOMTOM_A_RX_LINE_4_DAC_CTL] = 1,
	[TOMTOM_A_RX_LINE_4_STATUS] = 1,
	[TOMTOM_A_RX_LINE_CNP_DBG] = 1,
	[TOMTOM_A_SPKR_DRV1_EN] = 1,
	[TOMTOM_A_SPKR_DRV1_GAIN] = 1,
	[TOMTOM_A_SPKR_DRV1_DAC_CTL] = 1,
	[TOMTOM_A_SPKR_DRV1_OCP_CTL] = 1,
	[TOMTOM_A_SPKR_DRV1_CLIP_DET] = 1,
	[TOMTOM_A_SPKR_DRV1_IEC] = 1,
	[TOMTOM_A_SPKR_DRV1_DBG_DAC] = 1,
	[TOMTOM_A_SPKR_DRV1_DBG_PA] = 1,
	[TOMTOM_A_SPKR_DRV1_DBG_PWRSTG] = 1,
	[TOMTOM_A_SPKR_DRV1_BIAS_LDO] = 1,
	[TOMTOM_A_SPKR_DRV1_BIAS_INT] = 1,
	[TOMTOM_A_SPKR_DRV1_BIAS_PA] = 1,
	[TOMTOM_A_SPKR_DRV1_STATUS_OCP] = 1,
	[TOMTOM_A_SPKR_DRV1_STATUS_PA] = 1,
	[TOMTOM_A_SPKR1_PROT_EN] = 1,
	[TOMTOM_A_SPKR1_PROT_ADC_TEST_EN] = 1,
	[TOMTOM_A_SPKR1_PROT_ATEST] = 1,
	[TOMTOM_A_SPKR1_PROT_LDO_CTRL] = 1,
	[TOMTOM_A_SPKR1_PROT_ISENSE_CTRL] = 1,
	[TOMTOM_A_SPKR1_PROT_VSENSE_CTRL] = 1,
	[TOMTOM_A_SPKR2_PROT_EN] = 1,
	[TOMTOM_A_SPKR2_PROT_ADC_TEST_EN] = 1,
	[TOMTOM_A_SPKR2_PROT_ATEST] = 1,
	[TOMTOM_A_SPKR2_PROT_LDO_CTRL] = 1,
	[TOMTOM_A_SPKR2_PROT_ISENSE_CTRL] = 1,
	[TOMTOM_A_SPKR2_PROT_VSENSE_CTRL] = 1,
	[TOMTOM_A_MBHC_HPH] = 1,
	[TOMTOM_A_CDC_ANC1_B1_CTL] = 1,
	[TOMTOM_A_CDC_ANC2_B1_CTL] = 1,
	[TOMTOM_A_CDC_ANC1_SHIFT] = 1,
	[TOMTOM_A_CDC_ANC2_SHIFT] = 1,
	[TOMTOM_A_CDC_ANC1_IIR_B1_CTL] = 1,
	[TOMTOM_A_CDC_ANC2_IIR_B1_CTL] = 1,
	[TOMTOM_A_CDC_ANC1_IIR_B2_CTL] = 1,
	[TOMTOM_A_CDC_ANC2_IIR_B2_CTL] = 1,
	[TOMTOM_A_CDC_ANC1_IIR_B3_CTL] = 1,
	[TOMTOM_A_CDC_ANC2_IIR_B3_CTL] = 1,
	[TOMTOM_A_CDC_ANC1_LPF_B1_CTL] = 1,
	[TOMTOM_A_CDC_ANC2_LPF_B1_CTL] = 1,
	[TOMTOM_A_CDC_ANC1_LPF_B2_CTL] = 1,
	[TOMTOM_A_CDC_ANC2_LPF_B2_CTL] = 1,
	[TOMTOM_A_CDC_ANC1_SPARE] = 1,
	[TOMTOM_A_CDC_ANC2_SPARE] = 1,
	[TOMTOM_A_CDC_ANC1_SMLPF_CTL] = 1,
	[TOMTOM_A_CDC_ANC2_SMLPF_CTL] = 1,
	[TOMTOM_A_CDC_ANC1_DCFLT_CTL] = 1,
	[TOMTOM_A_CDC_ANC2_DCFLT_CTL] = 1,
	[TOMTOM_A_CDC_ANC1_GAIN_CTL] = 1,
	[TOMTOM_A_CDC_ANC2_GAIN_CTL] = 1,
	[TOMTOM_A_CDC_ANC1_B2_CTL] = 1,
	[TOMTOM_A_CDC_ANC2_B2_CTL] = 1,
	[TOMTOM_A_CDC_TX1_VOL_CTL_TIMER] = 1,
	[TOMTOM_A_CDC_TX2_VOL_CTL_TIMER] = 1,
	[TOMTOM_A_CDC_TX3_VOL_CTL_TIMER] = 1,
	[TOMTOM_A_CDC_TX4_VOL_CTL_TIMER] = 1,
	[TOMTOM_A_CDC_TX5_VOL_CTL_TIMER] = 1,
	[TOMTOM_A_CDC_TX6_VOL_CTL_TIMER] = 1,
	[TOMTOM_A_CDC_TX7_VOL_CTL_TIMER] = 1,
	[TOMTOM_A_CDC_TX8_VOL_CTL_TIMER] = 1,
	[TOMTOM_A_CDC_TX9_VOL_CTL_TIMER] = 1,
	[TOMTOM_A_CDC_TX10_VOL_CTL_TIMER] = 1,
	[TOMTOM_A_CDC_TX1_VOL_CTL_GAIN] = 1,
	[TOMTOM_A_CDC_TX2_VOL_CTL_GAIN] = 1,
	[TOMTOM_A_CDC_TX3_VOL_CTL_GAIN] = 1,
	[TOMTOM_A_CDC_TX4_VOL_CTL_GAIN] = 1,
	[TOMTOM_A_CDC_TX5_VOL_CTL_GAIN] = 1,
	[TOMTOM_A_CDC_TX6_VOL_CTL_GAIN] = 1,
	[TOMTOM_A_CDC_TX7_VOL_CTL_GAIN] = 1,
	[TOMTOM_A_CDC_TX8_VOL_CTL_GAIN] = 1,
	[TOMTOM_A_CDC_TX9_VOL_CTL_GAIN] = 1,
	[TOMTOM_A_CDC_TX10_VOL_CTL_GAIN] = 1,
	[TOMTOM_A_CDC_TX1_VOL_CTL_CFG] = 1,
	[TOMTOM_A_CDC_TX2_VOL_CTL_CFG] = 1,
	[TOMTOM_A_CDC_TX3_VOL_CTL_CFG] = 1,
	[TOMTOM_A_CDC_TX4_VOL_CTL_CFG] = 1,
	[TOMTOM_A_CDC_TX5_VOL_CTL_CFG] = 1,
	[TOMTOM_A_CDC_TX6_VOL_CTL_CFG] = 1,
	[TOMTOM_A_CDC_TX7_VOL_CTL_CFG] = 1,
	[TOMTOM_A_CDC_TX8_VOL_CTL_CFG] = 1,
	[TOMTOM_A_CDC_TX9_VOL_CTL_CFG] = 1,
	[TOMTOM_A_CDC_TX10_VOL_CTL_CFG] = 1,
	[TOMTOM_A_CDC_TX1_MUX_CTL] = 1,
	[TOMTOM_A_CDC_TX2_MUX_CTL] = 1,
	[TOMTOM_A_CDC_TX3_MUX_CTL] = 1,
	[TOMTOM_A_CDC_TX4_MUX_CTL] = 1,
	[TOMTOM_A_CDC_TX5_MUX_CTL] = 1,
	[TOMTOM_A_CDC_TX6_MUX_CTL] = 1,
	[TOMTOM_A_CDC_TX7_MUX_CTL] = 1,
	[TOMTOM_A_CDC_TX8_MUX_CTL] = 1,
	[TOMTOM_A_CDC_TX9_MUX_CTL] = 1,
	[TOMTOM_A_CDC_TX10_MUX_CTL] = 1,
	[TOMTOM_A_CDC_TX1_CLK_FS_CTL] = 1,
	[TOMTOM_A_CDC_TX2_CLK_FS_CTL] = 1,
	[TOMTOM_A_CDC_TX3_CLK_FS_CTL] = 1,
	[TOMTOM_A_CDC_TX4_CLK_FS_CTL] = 1,
	[TOMTOM_A_CDC_TX5_CLK_FS_CTL] = 1,
	[TOMTOM_A_CDC_TX6_CLK_FS_CTL] = 1,
	[TOMTOM_A_CDC_TX7_CLK_FS_CTL] = 1,
	[TOMTOM_A_CDC_TX8_CLK_FS_CTL] = 1,
	[TOMTOM_A_CDC_TX9_CLK_FS_CTL] = 1,
	[TOMTOM_A_CDC_TX10_CLK_FS_CTL] = 1,
	[TOMTOM_A_CDC_TX1_DMIC_CTL] = 1,
	[TOMTOM_A_CDC_TX2_DMIC_CTL] = 1,
	[TOMTOM_A_CDC_TX3_DMIC_CTL] = 1,
	[TOMTOM_A_CDC_TX4_DMIC_CTL] = 1,
	[TOMTOM_A_CDC_TX5_DMIC_CTL] = 1,
	[TOMTOM_A_CDC_TX6_DMIC_CTL] = 1,
	[TOMTOM_A_CDC_TX7_DMIC_CTL] = 1,
	[TOMTOM_A_CDC_TX8_DMIC_CTL] = 1,
	[TOMTOM_A_CDC_TX9_DMIC_CTL] = 1,
	[TOMTOM_A_CDC_TX10_DMIC_CTL] = 1,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL0] = 1,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL1] = 1,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL2] = 1,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL3] = 1,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL4] = 1,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL5] = 1,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL6] = 1,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL7] = 1,
	[TOMTOM_A_CDC_DEBUG_B1_CTL] = 1,
	[TOMTOM_A_CDC_DEBUG_B2_CTL] = 1,
	[TOMTOM_A_CDC_DEBUG_B3_CTL] = 1,
	[TOMTOM_A_CDC_DEBUG_B4_CTL] = 1,
	[TOMTOM_A_CDC_DEBUG_B5_CTL] = 1,
	[TOMTOM_A_CDC_DEBUG_B6_CTL] = 1,
	[TOMTOM_A_CDC_DEBUG_B7_CTL] = 1,
	[TOMTOM_A_CDC_SRC1_PDA_CFG] = 1,
	[TOMTOM_A_CDC_SRC2_PDA_CFG] = 1,
	[TOMTOM_A_CDC_SRC1_FS_CTL] = 1,
	[TOMTOM_A_CDC_SRC2_FS_CTL] = 1,
	[TOMTOM_A_CDC_RX1_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX2_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX3_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX4_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX5_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX6_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX7_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX1_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX2_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX3_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX4_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX5_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX6_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX7_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX1_B3_CTL] = 1,
	[TOMTOM_A_CDC_RX2_B3_CTL] = 1,
	[TOMTOM_A_CDC_RX3_B3_CTL] = 1,
	[TOMTOM_A_CDC_RX4_B3_CTL] = 1,
	[TOMTOM_A_CDC_RX5_B3_CTL] = 1,
	[TOMTOM_A_CDC_RX6_B3_CTL] = 1,
	[TOMTOM_A_CDC_RX7_B3_CTL] = 1,
	[TOMTOM_A_CDC_RX1_B4_CTL] = 1,
	[TOMTOM_A_CDC_RX2_B4_CTL] = 1,
	[TOMTOM_A_CDC_RX3_B4_CTL] = 1,
	[TOMTOM_A_CDC_RX4_B4_CTL] = 1,
	[TOMTOM_A_CDC_RX5_B4_CTL] = 1,
	[TOMTOM_A_CDC_RX6_B4_CTL] = 1,
	[TOMTOM_A_CDC_RX7_B4_CTL] = 1,
	[TOMTOM_A_CDC_RX1_B5_CTL] = 1,
	[TOMTOM_A_CDC_RX2_B5_CTL] = 1,
	[TOMTOM_A_CDC_RX3_B5_CTL] = 1,
	[TOMTOM_A_CDC_RX4_B5_CTL] = 1,
	[TOMTOM_A_CDC_RX5_B5_CTL] = 1,
	[TOMTOM_A_CDC_RX6_B5_CTL] = 1,
	[TOMTOM_A_CDC_RX7_B5_CTL] = 1,
	[TOMTOM_A_CDC_RX1_B6_CTL] = 1,
	[TOMTOM_A_CDC_RX2_B6_CTL] = 1,
	[TOMTOM_A_CDC_RX3_B6_CTL] = 1,
	[TOMTOM_A_CDC_RX4_B6_CTL] = 1,
	[TOMTOM_A_CDC_RX5_B6_CTL] = 1,
	[TOMTOM_A_CDC_RX6_B6_CTL] = 1,
	[TOMTOM_A_CDC_RX7_B6_CTL] = 1,
	[TOMTOM_A_CDC_RX1_VOL_CTL_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX2_VOL_CTL_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX3_VOL_CTL_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX4_VOL_CTL_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX5_VOL_CTL_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX6_VOL_CTL_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX7_VOL_CTL_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX1_VOL_CTL_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX2_VOL_CTL_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX3_VOL_CTL_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX4_VOL_CTL_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX5_VOL_CTL_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX6_VOL_CTL_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX7_VOL_CTL_B2_CTL] = 1,
	[TOMTOM_A_CDC_VBAT_CFG] = 1,
	[TOMTOM_A_CDC_VBAT_ADC_CAL1] = 1,
	[TOMTOM_A_CDC_VBAT_ADC_CAL2] = 1,
	[TOMTOM_A_CDC_VBAT_ADC_CAL3] = 1,
	[TOMTOM_A_CDC_VBAT_PK_EST1] = 1,
	[TOMTOM_A_CDC_VBAT_PK_EST2] = 1,
	[TOMTOM_A_CDC_VBAT_PK_EST3] = 1,
	[TOMTOM_A_CDC_VBAT_RF_PROC1] = 1,
	[TOMTOM_A_CDC_VBAT_RF_PROC2] = 1,
	[TOMTOM_A_CDC_VBAT_TAC1] = 1,
	[TOMTOM_A_CDC_VBAT_TAC2] = 1,
	[TOMTOM_A_CDC_VBAT_TAC3] = 1,
	[TOMTOM_A_CDC_VBAT_TAC4] = 1,
	[TOMTOM_A_CDC_VBAT_GAIN_UPD1] = 1,
	[TOMTOM_A_CDC_VBAT_GAIN_UPD2] = 1,
	[TOMTOM_A_CDC_VBAT_GAIN_UPD3] = 1,
	[TOMTOM_A_CDC_VBAT_GAIN_UPD4] = 1,
	[TOMTOM_A_CDC_VBAT_DEBUG1] = 1,
	[TOMTOM_A_CDC_VBAT_GAIN_UPD_MON] = 0,
	[TOMTOM_A_CDC_VBAT_GAIN_MON_VAL] = 1,
	[TOMTOM_A_CDC_CLK_ANC_RESET_CTL] = 1,
	[TOMTOM_A_CDC_CLK_RX_RESET_CTL] = 1,
	[TOMTOM_A_CDC_CLK_TX_RESET_B1_CTL] = 1,
	[TOMTOM_A_CDC_CLK_TX_RESET_B2_CTL] = 1,
	[TOMTOM_A_CDC_CLK_RX_I2S_CTL] = 1,
	[TOMTOM_A_CDC_CLK_TX_I2S_CTL] = 1,
	[TOMTOM_A_CDC_CLK_OTHR_RESET_B1_CTL] = 1,
	[TOMTOM_A_CDC_CLK_OTHR_RESET_B2_CTL] = 1,
	[TOMTOM_A_CDC_CLK_TX_CLK_EN_B1_CTL] = 1,
	[TOMTOM_A_CDC_CLK_TX_CLK_EN_B2_CTL] = 1,
	[TOMTOM_A_CDC_CLK_OTHR_CTL] = 1,
	[TOMTOM_A_CDC_CLK_ANC_CLK_EN_CTL] = 1,
	[TOMTOM_A_CDC_CLK_RX_B1_CTL] = 1,
	[TOMTOM_A_CDC_CLK_RX_B2_CTL] = 1,
	[TOMTOM_A_CDC_CLK_MCLK_CTL] = 1,
	[TOMTOM_A_CDC_CLK_PDM_CTL] = 1,
	[TOMTOM_A_CDC_CLK_SD_CTL] = 1,
	[TOMTOM_A_CDC_CLSH_B1_CTL] = 1,
	[TOMTOM_A_CDC_CLSH_B2_CTL] = 1,
	[TOMTOM_A_CDC_CLSH_B3_CTL] = 1,
	[TOMTOM_A_CDC_CLSH_BUCK_NCP_VARS] = 1,
	[TOMTOM_A_CDC_CLSH_IDLE_HPH_THSD] = 1,
	[TOMTOM_A_CDC_CLSH_IDLE_EAR_THSD] = 1,
	[TOMTOM_A_CDC_CLSH_FCLKONLY_HPH_THSD] = 1,
	[TOMTOM_A_CDC_CLSH_FCLKONLY_EAR_THSD] = 1,
	[TOMTOM_A_CDC_CLSH_K_ADDR] = 1,
	[TOMTOM_A_CDC_CLSH_K_DATA] = 1,
	[TOMTOM_A_CDC_CLSH_I_PA_FACT_HPH_L] = 1,
	[TOMTOM_A_CDC_CLSH_I_PA_FACT_HPH_U] = 1,
	[TOMTOM_A_CDC_CLSH_I_PA_FACT_EAR_L] = 1,
	[TOMTOM_A_CDC_CLSH_I_PA_FACT_EAR_U] = 1,
	[TOMTOM_A_CDC_CLSH_V_PA_HD_EAR] = 1,
	[TOMTOM_A_CDC_CLSH_V_PA_HD_HPH] = 1,
	[TOMTOM_A_CDC_CLSH_V_PA_MIN_EAR] = 1,
	[TOMTOM_A_CDC_CLSH_V_PA_MIN_HPH] = 1,
	[TOMTOM_A_CDC_IIR1_GAIN_B1_CTL] = 1,
	[TOMTOM_A_CDC_IIR2_GAIN_B1_CTL] = 1,
	[TOMTOM_A_CDC_IIR1_GAIN_B2_CTL] = 1,
	[TOMTOM_A_CDC_IIR2_GAIN_B2_CTL] = 1,
	[TOMTOM_A_CDC_IIR1_GAIN_B3_CTL] = 1,
	[TOMTOM_A_CDC_IIR2_GAIN_B3_CTL] = 1,
	[TOMTOM_A_CDC_IIR1_GAIN_B4_CTL] = 1,
	[TOMTOM_A_CDC_IIR2_GAIN_B4_CTL] = 1,
	[TOMTOM_A_CDC_IIR1_GAIN_B5_CTL] = 1,
	[TOMTOM_A_CDC_IIR2_GAIN_B5_CTL] = 1,
	[TOMTOM_A_CDC_IIR1_GAIN_B6_CTL] = 1,
	[TOMTOM_A_CDC_IIR2_GAIN_B6_CTL] = 1,
	[TOMTOM_A_CDC_IIR1_GAIN_B7_CTL] = 1,
	[TOMTOM_A_CDC_IIR2_GAIN_B7_CTL] = 1,
	[TOMTOM_A_CDC_IIR1_GAIN_B8_CTL] = 1,
	[TOMTOM_A_CDC_IIR2_GAIN_B8_CTL] = 1,
	[TOMTOM_A_CDC_IIR1_CTL] = 1,
	[TOMTOM_A_CDC_IIR2_CTL] = 1,
	[TOMTOM_A_CDC_IIR1_GAIN_TIMER_CTL] = 1,
	[TOMTOM_A_CDC_IIR2_GAIN_TIMER_CTL] = 1,
	[TOMTOM_A_CDC_IIR1_COEF_B1_CTL] = 1,
	[TOMTOM_A_CDC_IIR2_COEF_B1_CTL] = 1,
	[TOMTOM_A_CDC_IIR1_COEF_B2_CTL] = 1,
	[TOMTOM_A_CDC_IIR2_COEF_B2_CTL] = 1,
	[TOMTOM_A_CDC_TOP_GAIN_UPDATE] = 1,
	[TOMTOM_A_CDC_PA_RAMP_B1_CTL] = 1,
	[TOMTOM_A_CDC_PA_RAMP_B2_CTL] = 1,
	[TOMTOM_A_CDC_PA_RAMP_B3_CTL] = 1,
	[TOMTOM_A_CDC_PA_RAMP_B4_CTL] = 1,
	[TOMTOM_A_CDC_SPKR_CLIPDET_B1_CTL] = 1,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_B1_CTL] = 1,
	[TOMTOM_A_CDC_COMP0_B1_CTL] = 1,
	[TOMTOM_A_CDC_COMP1_B1_CTL] = 1,
	[TOMTOM_A_CDC_COMP2_B1_CTL] = 1,
	[TOMTOM_A_CDC_COMP0_B2_CTL] = 1,
	[TOMTOM_A_CDC_COMP1_B2_CTL] = 1,
	[TOMTOM_A_CDC_COMP2_B2_CTL] = 1,
	[TOMTOM_A_CDC_COMP0_B3_CTL] = 1,
	[TOMTOM_A_CDC_COMP1_B3_CTL] = 1,
	[TOMTOM_A_CDC_COMP2_B3_CTL] = 1,
	[TOMTOM_A_CDC_COMP0_B4_CTL] = 1,
	[TOMTOM_A_CDC_COMP1_B4_CTL] = 1,
	[TOMTOM_A_CDC_COMP2_B4_CTL] = 1,
	[TOMTOM_A_CDC_COMP0_B5_CTL] = 1,
	[TOMTOM_A_CDC_COMP1_B5_CTL] = 1,
	[TOMTOM_A_CDC_COMP2_B5_CTL] = 1,
	[TOMTOM_A_CDC_COMP0_B6_CTL] = 1,
	[TOMTOM_A_CDC_COMP1_B6_CTL] = 1,
	[TOMTOM_A_CDC_COMP2_B6_CTL] = 1,
	[TOMTOM_A_CDC_COMP0_SHUT_DOWN_STATUS] = 1,
	[TOMTOM_A_CDC_COMP1_SHUT_DOWN_STATUS] = 1,
	[TOMTOM_A_CDC_COMP2_SHUT_DOWN_STATUS] = 1,
	[TOMTOM_A_CDC_COMP0_FS_CFG] = 1,
	[TOMTOM_A_CDC_COMP1_FS_CFG] = 1,
	[TOMTOM_A_CDC_COMP2_FS_CFG] = 1,
	[TOMTOM_A_CDC_CONN_RX1_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX1_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX1_B3_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX2_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX2_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX2_B3_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX3_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX3_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX4_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX4_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX5_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX5_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX6_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX6_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX7_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX7_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX7_B3_CTL] = 1,
	[TOMTOM_A_CDC_CONN_ANC_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_ANC_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_B3_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_B4_CTL] = 1,
	[TOMTOM_A_CDC_CONN_EQ1_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_EQ1_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_EQ1_B3_CTL] = 1,
	[TOMTOM_A_CDC_CONN_EQ1_B4_CTL] = 1,
	[TOMTOM_A_CDC_CONN_EQ2_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_EQ2_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_EQ2_B3_CTL] = 1,
	[TOMTOM_A_CDC_CONN_EQ2_B4_CTL] = 1,
	[TOMTOM_A_CDC_CONN_SRC1_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_SRC1_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_SRC2_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_SRC2_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_SB_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_SB_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_SB_B3_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_SB_B4_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_SB_B5_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_SB_B6_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_SB_B7_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_SB_B8_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_SB_B9_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_SB_B10_CTL] = 1,
	[TOMTOM_A_CDC_CONN_TX_SB_B11_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX_SB_B1_CTL] = 1,
	[TOMTOM_A_CDC_CONN_RX_SB_B2_CTL] = 1,
	[TOMTOM_A_CDC_CONN_CLSH_CTL] = 1,
	[TOMTOM_A_CDC_CONN_MISC] = 1,
	[TOMTOM_A_CDC_CONN_RX8_B1_CTL] = 1,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR_B1_CTL] = 1,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR_CLIP_LEVEL_ADJUST] = 1,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR_MIN_CLIP_THRESHOLD] = 1,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR_THRESHOLD_STATUS] = 1,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR_SAMPLE_MARK] = 1,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR_BOOST_GATING] = 1,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR2_B1_CTL] = 1,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR2_CLIP_LEVEL_ADJUST] = 1,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR2_MIN_CLIP_THRESHOLD] = 1,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR2_THRESHOLD_STATUS] = 1,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR2_SAMPLE_MARK] = 1,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR2_BOOST_GATING] = 1,
	[TOMTOM_A_CDC_MBHC_EN_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_FIR_B1_CFG] = 1,
	[TOMTOM_A_CDC_MBHC_FIR_B2_CFG] = 1,
	[TOMTOM_A_CDC_MBHC_TIMER_B1_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_TIMER_B2_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_TIMER_B3_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_TIMER_B4_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_TIMER_B5_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_TIMER_B6_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_B1_STATUS] = 1,
	[TOMTOM_A_CDC_MBHC_B2_STATUS] = 1,
	[TOMTOM_A_CDC_MBHC_B3_STATUS] = 1,
	[TOMTOM_A_CDC_MBHC_B4_STATUS] = 1,
	[TOMTOM_A_CDC_MBHC_B5_STATUS] = 1,
	[TOMTOM_A_CDC_MBHC_B1_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_B2_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_VOLT_B1_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_VOLT_B2_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_VOLT_B3_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_VOLT_B4_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_VOLT_B5_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_VOLT_B6_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_VOLT_B7_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_VOLT_B8_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_VOLT_B9_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_VOLT_B10_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_VOLT_B11_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_VOLT_B12_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_CLK_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_INT_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_DEBUG_CTL] = 1,
	[TOMTOM_A_CDC_MBHC_SPARE] = 1,
	[TOMTOM_A_CDC_RX8_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX8_B2_CTL] = 1,
	[TOMTOM_A_CDC_RX8_B3_CTL] = 1,
	[TOMTOM_A_CDC_RX8_B4_CTL] = 1,
	[TOMTOM_A_CDC_RX8_B5_CTL] = 1,
	[TOMTOM_A_CDC_RX8_B6_CTL] = 1,
	[TOMTOM_A_CDC_RX8_VOL_CTL_B1_CTL] = 1,
	[TOMTOM_A_CDC_RX8_VOL_CTL_B2_CTL] = 1,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL0] = 1,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL1] = 1,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL2] = 1,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL3] = 1,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL4] = 1,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL5] = 1,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL6] = 1,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL7] = 1,
	[TOMTOM_A_CDC_BOOST_MODE_CTL] = 1,
	[TOMTOM_A_CDC_BOOST_THRESHOLD] = 1,
	[TOMTOM_A_CDC_BOOST_TAP_SEL] = 1,
	[TOMTOM_A_CDC_BOOST_HOLD_TIME] = 1,
	[TOMTOM_A_CDC_BOOST_TRGR_EN] = 1,
};

const u8 tomtom_reset_reg_defaults[TOMTOM_CACHE_SIZE] = {
	[TOMTOM_A_CHIP_CTL] = TOMTOM_A_CHIP_CTL__POR,
	[TOMTOM_A_CHIP_STATUS] = TOMTOM_A_CHIP_STATUS__POR,
	[TOMTOM_A_CHIP_ID_BYTE_0] = TOMTOM_A_CHIP_ID_BYTE_0__POR,
	[TOMTOM_A_CHIP_ID_BYTE_1] = TOMTOM_A_CHIP_ID_BYTE_1__POR,
	[TOMTOM_A_CHIP_ID_BYTE_2] = TOMTOM_A_CHIP_ID_BYTE_2__POR,
	[TOMTOM_A_CHIP_ID_BYTE_3] = TOMTOM_A_CHIP_ID_BYTE_3__POR,
	[TOMTOM_A_CHIP_I2C_SLAVE_ID] = TOMTOM_A_CHIP_I2C_SLAVE_ID__POR,
	[TOMTOM_A_SLAVE_ID_1] = TOMTOM_A_SLAVE_ID_1__POR,
	[TOMTOM_A_SLAVE_ID_2] = TOMTOM_A_SLAVE_ID_2__POR,
	[TOMTOM_A_SLAVE_ID_3] = TOMTOM_A_SLAVE_ID_3__POR,
	[TOMTOM_A_PIN_CTL_OE0] = TOMTOM_A_PIN_CTL_OE0__POR,
	[TOMTOM_A_PIN_CTL_OE1] = TOMTOM_A_PIN_CTL_OE1__POR,
	[TOMTOM_A_PIN_CTL_OE2] = TOMTOM_A_PIN_CTL_OE2__POR,
	[TOMTOM_A_PIN_CTL_DATA0] = TOMTOM_A_PIN_CTL_DATA0__POR,
	[TOMTOM_A_PIN_CTL_DATA1] = TOMTOM_A_PIN_CTL_DATA1__POR,
	[TOMTOM_A_PIN_CTL_DATA2] = TOMTOM_A_PIN_CTL_DATA2__POR,
	[TOMTOM_A_HDRIVE_GENERIC] = TOMTOM_A_HDRIVE_GENERIC__POR,
	[TOMTOM_A_HDRIVE_OVERRIDE] = TOMTOM_A_HDRIVE_OVERRIDE__POR,
	[TOMTOM_A_ANA_CSR_WAIT_STATE] = TOMTOM_A_ANA_CSR_WAIT_STATE__POR,
	[TOMTOM_A_PROCESS_MONITOR_CTL0] = TOMTOM_A_PROCESS_MONITOR_CTL0__POR,
	[TOMTOM_A_PROCESS_MONITOR_CTL1] = TOMTOM_A_PROCESS_MONITOR_CTL1__POR,
	[TOMTOM_A_PROCESS_MONITOR_CTL2] = TOMTOM_A_PROCESS_MONITOR_CTL2__POR,
	[TOMTOM_A_PROCESS_MONITOR_CTL3] = TOMTOM_A_PROCESS_MONITOR_CTL3__POR,
	[TOMTOM_A_QFUSE_CTL] = TOMTOM_A_QFUSE_CTL__POR,
	[TOMTOM_A_QFUSE_STATUS] = TOMTOM_A_QFUSE_STATUS__POR,
	[TOMTOM_A_QFUSE_DATA_OUT0] = TOMTOM_A_QFUSE_DATA_OUT0__POR,
	[TOMTOM_A_QFUSE_DATA_OUT1] = TOMTOM_A_QFUSE_DATA_OUT1__POR,
	[TOMTOM_A_QFUSE_DATA_OUT2] = TOMTOM_A_QFUSE_DATA_OUT2__POR,
	[TOMTOM_A_QFUSE_DATA_OUT3] = TOMTOM_A_QFUSE_DATA_OUT3__POR,
	[TOMTOM_A_QFUSE_DATA_OUT4] = TOMTOM_A_QFUSE_DATA_OUT4__POR,
	[TOMTOM_A_QFUSE_DATA_OUT5] = TOMTOM_A_QFUSE_DATA_OUT5__POR,
	[TOMTOM_A_QFUSE_DATA_OUT6] = TOMTOM_A_QFUSE_DATA_OUT6__POR,
	[TOMTOM_A_QFUSE_DATA_OUT7] = TOMTOM_A_QFUSE_DATA_OUT7__POR,
	[TOMTOM_A_CDC_CTL] = TOMTOM_A_CDC_CTL__POR,
	[TOMTOM_A_LEAKAGE_CTL] = TOMTOM_A_LEAKAGE_CTL__POR,
	[TOMTOM_A_SVASS_MEM_PTR0] = TOMTOM_A_SVASS_MEM_PTR0__POR,
	[TOMTOM_A_SVASS_MEM_PTR1] = TOMTOM_A_SVASS_MEM_PTR1__POR,
	[TOMTOM_A_SVASS_MEM_PTR2] = TOMTOM_A_SVASS_MEM_PTR2__POR,
	[TOMTOM_A_SVASS_MEM_CTL] = TOMTOM_A_SVASS_MEM_CTL__POR,
	[TOMTOM_A_SVASS_MEM_BANK] = TOMTOM_A_SVASS_MEM_BANK__POR,
	[TOMTOM_A_DMIC_B1_CTL] = TOMTOM_A_DMIC_B1_CTL__POR,
	[TOMTOM_A_DMIC_B2_CTL] = TOMTOM_A_DMIC_B2_CTL__POR,
	[TOMTOM_A_SVASS_CLKRST_CTL] = TOMTOM_A_SVASS_CLKRST_CTL__POR,
	[TOMTOM_A_SVASS_CPAR_CFG] = TOMTOM_A_SVASS_CPAR_CFG__POR,
	[TOMTOM_A_SVASS_BUF_RDY_INT_PERIOD] =
				TOMTOM_A_SVASS_BUF_RDY_INT_PERIOD__POR,
	[TOMTOM_A_SVASS_CPAR_WDOG_CFG] = TOMTOM_A_SVASS_CPAR_WDOG_CFG__POR,
	[TOMTOM_A_SVASS_CFG] = TOMTOM_A_SVASS_CFG__POR,
	[TOMTOM_A_SVASS_SPE_CFG] = TOMTOM_A_SVASS_SPE_CFG__POR,
	[TOMTOM_A_SVASS_STATUS] = TOMTOM_A_SVASS_STATUS__POR,
	[TOMTOM_A_SVASS_INT_MASK] = TOMTOM_A_SVASS_INT_MASK__POR,
	[TOMTOM_A_SVASS_INT_STATUS] = TOMTOM_A_SVASS_INT_STATUS__POR,
	[TOMTOM_A_SVASS_INT_CLR] = TOMTOM_A_SVASS_INT_CLR__POR,
	[TOMTOM_A_SVASS_DEBUG] = TOMTOM_A_SVASS_DEBUG__POR,
	[TOMTOM_A_SVASS_SPE_BKUP_INT] = TOMTOM_A_SVASS_SPE_BKUP_INT__POR,
	[TOMTOM_A_SVASS_MEM_ACC] = TOMTOM_A_SVASS_MEM_ACC__POR,
	[TOMTOM_A_MEM_LEAKAGE_CTL] = TOMTOM_A_MEM_LEAKAGE_CTL__POR,
	[TOMTOM_A_SVASS_SPE_INBOX_TRG] = TOMTOM_A_SVASS_SPE_INBOX_TRG__POR,
	[TOMTOM_A_SVASS_SPE_INBOX_0] = TOMTOM_A_SVASS_SPE_INBOX_0__POR,
	[TOMTOM_A_SVASS_SPE_INBOX_1] = TOMTOM_A_SVASS_SPE_INBOX_1__POR,
	[TOMTOM_A_SVASS_SPE_INBOX_2] = TOMTOM_A_SVASS_SPE_INBOX_2__POR,
	[TOMTOM_A_SVASS_SPE_INBOX_3] = TOMTOM_A_SVASS_SPE_INBOX_3__POR,
	[TOMTOM_A_SVASS_SPE_INBOX_4] = TOMTOM_A_SVASS_SPE_INBOX_4__POR,
	[TOMTOM_A_SVASS_SPE_INBOX_5] = TOMTOM_A_SVASS_SPE_INBOX_5__POR,
	[TOMTOM_A_SVASS_SPE_INBOX_6] = TOMTOM_A_SVASS_SPE_INBOX_6__POR,
	[TOMTOM_A_SVASS_SPE_INBOX_7] = TOMTOM_A_SVASS_SPE_INBOX_7__POR,
	[TOMTOM_A_SVASS_SPE_INBOX_8] = TOMTOM_A_SVASS_SPE_INBOX_8__POR,
	[TOMTOM_A_SVASS_SPE_INBOX_9] = TOMTOM_A_SVASS_SPE_INBOX_9__POR,
	[TOMTOM_A_SVASS_SPE_INBOX_10] = TOMTOM_A_SVASS_SPE_INBOX_10__POR,
	[TOMTOM_A_SVASS_SPE_INBOX_11] = TOMTOM_A_SVASS_SPE_INBOX_11__POR,
	[TOMTOM_A_SVASS_SPE_OUTBOX_0] = TOMTOM_A_SVASS_SPE_OUTBOX_0__POR,
	[TOMTOM_A_SVASS_SPE_OUTBOX_1] = TOMTOM_A_SVASS_SPE_OUTBOX_1__POR,
	[TOMTOM_A_SVASS_SPE_OUTBOX_2] = TOMTOM_A_SVASS_SPE_OUTBOX_2__POR,
	[TOMTOM_A_SVASS_SPE_OUTBOX_3] = TOMTOM_A_SVASS_SPE_OUTBOX_3__POR,
	[TOMTOM_A_SVASS_SPE_OUTBOX_4] = TOMTOM_A_SVASS_SPE_OUTBOX_4__POR,
	[TOMTOM_A_SVASS_SPE_OUTBOX_5] = TOMTOM_A_SVASS_SPE_OUTBOX_5__POR,
	[TOMTOM_A_SVASS_SPE_OUTBOX_6] = TOMTOM_A_SVASS_SPE_OUTBOX_6__POR,
	[TOMTOM_A_SVASS_SPE_OUTBOX_7] = TOMTOM_A_SVASS_SPE_OUTBOX_7__POR,
	[TOMTOM_A_SVASS_SPE_OUTBOX_8] = TOMTOM_A_SVASS_SPE_OUTBOX_8__POR,
	[TOMTOM_A_SVASS_SPE_OUTBOX_9] = TOMTOM_A_SVASS_SPE_OUTBOX_9__POR,
	[TOMTOM_A_SVASS_SPE_OUTBOX_10] = TOMTOM_A_SVASS_SPE_OUTBOX_10__POR,
	[TOMTOM_A_SVASS_SPE_OUTBOX_11] = TOMTOM_A_SVASS_SPE_OUTBOX_11__POR,
	[TOMTOM_A_INTR_MODE] = TOMTOM_A_INTR_MODE__POR,
	[TOMTOM_A_INTR1_MASK0] = TOMTOM_A_INTR1_MASK0__POR,
	[TOMTOM_A_INTR1_MASK1] = TOMTOM_A_INTR1_MASK1__POR,
	[TOMTOM_A_INTR1_MASK2] = TOMTOM_A_INTR1_MASK2__POR,
	[TOMTOM_A_INTR1_MASK3] = TOMTOM_A_INTR1_MASK3__POR,
	[TOMTOM_A_INTR1_STATUS0] = TOMTOM_A_INTR1_STATUS0__POR,
	[TOMTOM_A_INTR1_STATUS1] = TOMTOM_A_INTR1_STATUS1__POR,
	[TOMTOM_A_INTR1_STATUS2] = TOMTOM_A_INTR1_STATUS2__POR,
	[TOMTOM_A_INTR1_STATUS3] = TOMTOM_A_INTR1_STATUS3__POR,
	[TOMTOM_A_INTR1_CLEAR0] = TOMTOM_A_INTR1_CLEAR0__POR,
	[TOMTOM_A_INTR1_CLEAR1] = TOMTOM_A_INTR1_CLEAR1__POR,
	[TOMTOM_A_INTR1_CLEAR2] = TOMTOM_A_INTR1_CLEAR2__POR,
	[TOMTOM_A_INTR1_CLEAR3] = TOMTOM_A_INTR1_CLEAR3__POR,
	[TOMTOM_A_INTR1_LEVEL0] = TOMTOM_A_INTR1_LEVEL0__POR,
	[TOMTOM_A_INTR1_LEVEL1] = TOMTOM_A_INTR1_LEVEL1__POR,
	[TOMTOM_A_INTR1_LEVEL2] = TOMTOM_A_INTR1_LEVEL2__POR,
	[TOMTOM_A_INTR1_LEVEL3] = TOMTOM_A_INTR1_LEVEL3__POR,
	[TOMTOM_A_INTR1_TEST0] = TOMTOM_A_INTR1_TEST0__POR,
	[TOMTOM_A_INTR1_TEST1] = TOMTOM_A_INTR1_TEST1__POR,
	[TOMTOM_A_INTR1_TEST2] = TOMTOM_A_INTR1_TEST2__POR,
	[TOMTOM_A_INTR1_TEST3] = TOMTOM_A_INTR1_TEST3__POR,
	[TOMTOM_A_INTR1_SET0] = TOMTOM_A_INTR1_SET0__POR,
	[TOMTOM_A_INTR1_SET1] = TOMTOM_A_INTR1_SET1__POR,
	[TOMTOM_A_INTR1_SET2] = TOMTOM_A_INTR1_SET2__POR,
	[TOMTOM_A_INTR1_SET3] = TOMTOM_A_INTR1_SET3__POR,
	[TOMTOM_A_INTR2_MASK0] = TOMTOM_A_INTR2_MASK0__POR,
	[TOMTOM_A_INTR2_STATUS0] = TOMTOM_A_INTR2_STATUS0__POR,
	[TOMTOM_A_INTR2_CLEAR0] = TOMTOM_A_INTR2_CLEAR0__POR,
	[TOMTOM_A_INTR2_LEVEL0] = TOMTOM_A_INTR2_LEVEL0__POR,
	[TOMTOM_A_INTR2_TEST0] = TOMTOM_A_INTR2_TEST0__POR,
	[TOMTOM_A_INTR2_SET0] = TOMTOM_A_INTR2_SET0__POR,
	[TOMTOM_A_CDC_TX_I2S_SCK_MODE] = TOMTOM_A_CDC_TX_I2S_SCK_MODE__POR,
	[TOMTOM_A_CDC_TX_I2S_WS_MODE] = TOMTOM_A_CDC_TX_I2S_WS_MODE__POR,
	[TOMTOM_A_CDC_DMIC_DATA0_MODE] = TOMTOM_A_CDC_DMIC_DATA0_MODE__POR,
	[TOMTOM_A_CDC_DMIC_CLK0_MODE] = TOMTOM_A_CDC_DMIC_CLK0_MODE__POR,
	[TOMTOM_A_CDC_DMIC_DATA1_MODE] = TOMTOM_A_CDC_DMIC_DATA1_MODE__POR,
	[TOMTOM_A_CDC_DMIC_CLK1_MODE] = TOMTOM_A_CDC_DMIC_CLK1_MODE__POR,
	[TOMTOM_A_CDC_RX_I2S_SCK_MODE] = TOMTOM_A_CDC_RX_I2S_SCK_MODE__POR,
	[TOMTOM_A_CDC_RX_I2S_WS_MODE] = TOMTOM_A_CDC_RX_I2S_WS_MODE__POR,
	[TOMTOM_A_CDC_DMIC_DATA2_MODE] = TOMTOM_A_CDC_DMIC_DATA2_MODE__POR,
	[TOMTOM_A_CDC_DMIC_CLK2_MODE] = TOMTOM_A_CDC_DMIC_CLK2_MODE__POR,
	[TOMTOM_A_CDC_INTR1_MODE] = TOMTOM_A_CDC_INTR1_MODE__POR,
	[TOMTOM_A_CDC_SB_NRZ_SEL_MODE] = TOMTOM_A_CDC_SB_NRZ_SEL_MODE__POR,
	[TOMTOM_A_CDC_INTR2_MODE] = TOMTOM_A_CDC_INTR2_MODE__POR,
	[TOMTOM_A_CDC_RF_PA_ON_MODE] = TOMTOM_A_CDC_RF_PA_ON_MODE__POR,
	[TOMTOM_A_CDC_BOOST_MODE] = TOMTOM_A_CDC_BOOST_MODE__POR,
	[TOMTOM_A_CDC_JTCK_MODE] = TOMTOM_A_CDC_JTCK_MODE__POR,
	[TOMTOM_A_CDC_JTDI_MODE] = TOMTOM_A_CDC_JTDI_MODE__POR,
	[TOMTOM_A_CDC_JTMS_MODE] = TOMTOM_A_CDC_JTMS_MODE__POR,
	[TOMTOM_A_CDC_JTDO_MODE] = TOMTOM_A_CDC_JTDO_MODE__POR,
	[TOMTOM_A_CDC_JTRST_MODE] = TOMTOM_A_CDC_JTRST_MODE__POR,
	[TOMTOM_A_CDC_BIST_MODE_MODE] = TOMTOM_A_CDC_BIST_MODE_MODE__POR,
	[TOMTOM_A_CDC_MAD_MAIN_CTL_1] = TOMTOM_A_CDC_MAD_MAIN_CTL_1__POR,
	[TOMTOM_A_CDC_MAD_MAIN_CTL_2] = TOMTOM_A_CDC_MAD_MAIN_CTL_2__POR,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_1] = TOMTOM_A_CDC_MAD_AUDIO_CTL_1__POR,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_2] = TOMTOM_A_CDC_MAD_AUDIO_CTL_2__POR,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_3] = TOMTOM_A_CDC_MAD_AUDIO_CTL_3__POR,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_4] = TOMTOM_A_CDC_MAD_AUDIO_CTL_4__POR,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_5] = TOMTOM_A_CDC_MAD_AUDIO_CTL_5__POR,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_6] = TOMTOM_A_CDC_MAD_AUDIO_CTL_6__POR,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_7] = TOMTOM_A_CDC_MAD_AUDIO_CTL_7__POR,
	[TOMTOM_A_CDC_MAD_AUDIO_CTL_8] = TOMTOM_A_CDC_MAD_AUDIO_CTL_8__POR,
	[TOMTOM_A_CDC_MAD_AUDIO_IIR_CTL_PTR] =
				TOMTOM_A_CDC_MAD_AUDIO_IIR_CTL_PTR__POR,
	[TOMTOM_A_CDC_MAD_AUDIO_IIR_CTL_VAL] =
				TOMTOM_A_CDC_MAD_AUDIO_IIR_CTL_VAL__POR,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_1] = TOMTOM_A_CDC_MAD_ULTR_CTL_1__POR,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_2] = TOMTOM_A_CDC_MAD_ULTR_CTL_2__POR,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_3] = TOMTOM_A_CDC_MAD_ULTR_CTL_3__POR,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_4] = TOMTOM_A_CDC_MAD_ULTR_CTL_4__POR,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_5] = TOMTOM_A_CDC_MAD_ULTR_CTL_5__POR,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_6] = TOMTOM_A_CDC_MAD_ULTR_CTL_6__POR,
	[TOMTOM_A_CDC_MAD_ULTR_CTL_7] = TOMTOM_A_CDC_MAD_ULTR_CTL_7__POR,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_1] = TOMTOM_A_CDC_MAD_BEACON_CTL_1__POR,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_2] = TOMTOM_A_CDC_MAD_BEACON_CTL_2__POR,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_3] = TOMTOM_A_CDC_MAD_BEACON_CTL_3__POR,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_4] = TOMTOM_A_CDC_MAD_BEACON_CTL_4__POR,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_5] = TOMTOM_A_CDC_MAD_BEACON_CTL_5__POR,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_6] = TOMTOM_A_CDC_MAD_BEACON_CTL_6__POR,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_7] = TOMTOM_A_CDC_MAD_BEACON_CTL_7__POR,
	[TOMTOM_A_CDC_MAD_BEACON_CTL_8] = TOMTOM_A_CDC_MAD_BEACON_CTL_8__POR,
	[TOMTOM_A_CDC_MAD_BEACON_IIR_CTL_PTR] =
				TOMTOM_A_CDC_MAD_BEACON_IIR_CTL_PTR__POR,
	[TOMTOM_A_CDC_MAD_BEACON_IIR_CTL_VAL] =
				TOMTOM_A_CDC_MAD_BEACON_IIR_CTL_VAL__POR,
	[TOMTOM_A_CDC_MAD_INP_SEL] = TOMTOM_A_CDC_MAD_INP_SEL__POR,
	[TOMTOM_A_BIAS_REF_CTL] = TOMTOM_A_BIAS_REF_CTL__POR,
	[TOMTOM_A_BIAS_CENTRAL_BG_CTL] = TOMTOM_A_BIAS_CENTRAL_BG_CTL__POR,
	[TOMTOM_A_BIAS_PRECHRG_CTL] = TOMTOM_A_BIAS_PRECHRG_CTL__POR,
	[TOMTOM_A_BIAS_CURR_CTL_1] = TOMTOM_A_BIAS_CURR_CTL_1__POR,
	[TOMTOM_A_BIAS_CURR_CTL_2] = TOMTOM_A_BIAS_CURR_CTL_2__POR,
	[TOMTOM_A_BIAS_OSC_BG_CTL] = TOMTOM_A_BIAS_OSC_BG_CTL__POR,
	[TOMTOM_A_CLK_BUFF_EN1] = TOMTOM_A_CLK_BUFF_EN1__POR,
	[TOMTOM_A_CLK_BUFF_EN2] = TOMTOM_A_CLK_BUFF_EN2__POR,
	[TOMTOM_A_LDO_L_MODE_1] = TOMTOM_A_LDO_L_MODE_1__POR,
	[TOMTOM_A_LDO_L_MODE_2] = TOMTOM_A_LDO_L_MODE_2__POR,
	[TOMTOM_A_LDO_L_CTRL_1] = TOMTOM_A_LDO_L_CTRL_1__POR,
	[TOMTOM_A_LDO_L_CTRL_2] = TOMTOM_A_LDO_L_CTRL_2__POR,
	[TOMTOM_A_LDO_L_CTRL_3] = TOMTOM_A_LDO_L_CTRL_3__POR,
	[TOMTOM_A_LDO_L_CTRL_4] = TOMTOM_A_LDO_L_CTRL_4__POR,
	[TOMTOM_A_LDO_H_MODE_1] = TOMTOM_A_LDO_H_MODE_1__POR,
	[TOMTOM_A_LDO_H_MODE_2] = TOMTOM_A_LDO_H_MODE_2__POR,
	[TOMTOM_A_LDO_H_LOOP_CTL] = TOMTOM_A_LDO_H_LOOP_CTL__POR,
	[TOMTOM_A_LDO_H_COMP_1] = TOMTOM_A_LDO_H_COMP_1__POR,
	[TOMTOM_A_LDO_H_COMP_2] = TOMTOM_A_LDO_H_COMP_2__POR,
	[TOMTOM_A_LDO_H_BIAS_1] = TOMTOM_A_LDO_H_BIAS_1__POR,
	[TOMTOM_A_LDO_H_BIAS_2] = TOMTOM_A_LDO_H_BIAS_2__POR,
	[TOMTOM_A_LDO_H_BIAS_3] = TOMTOM_A_LDO_H_BIAS_3__POR,
	[TOMTOM_A_VBAT_CLK] = TOMTOM_A_VBAT_CLK__POR,
	[TOMTOM_A_VBAT_LOOP] = TOMTOM_A_VBAT_LOOP__POR,
	[TOMTOM_A_VBAT_REF] = TOMTOM_A_VBAT_REF__POR,
	[TOMTOM_A_VBAT_ADC_TEST] = TOMTOM_A_VBAT_ADC_TEST__POR,
	[TOMTOM_A_VBAT_FE] = TOMTOM_A_VBAT_FE__POR,
	[TOMTOM_A_VBAT_BIAS_1] = TOMTOM_A_VBAT_BIAS_1__POR,
	[TOMTOM_A_VBAT_BIAS_2] = TOMTOM_A_VBAT_BIAS_2__POR,
	[TOMTOM_A_VBAT_ADC_DATA_MSB] = TOMTOM_A_VBAT_ADC_DATA_MSB__POR,
	[TOMTOM_A_VBAT_ADC_DATA_LSB] = TOMTOM_A_VBAT_ADC_DATA_LSB__POR,
	[TOMTOM_A_FLL_NREF] = TOMTOM_A_FLL_NREF__POR,
	[TOMTOM_A_FLL_KDCO_TUNE] = TOMTOM_A_FLL_KDCO_TUNE__POR,
	[TOMTOM_A_FLL_LOCK_THRESH] = TOMTOM_A_FLL_LOCK_THRESH__POR,
	[TOMTOM_A_FLL_LOCK_DET_COUNT] = TOMTOM_A_FLL_LOCK_DET_COUNT__POR,
	[TOMTOM_A_FLL_DAC_THRESHOLD] = TOMTOM_A_FLL_DAC_THRESHOLD__POR,
	[TOMTOM_A_FLL_TEST_DCO_FREERUN] = TOMTOM_A_FLL_TEST_DCO_FREERUN__POR,
	[TOMTOM_A_FLL_TEST_ENABLE] = TOMTOM_A_FLL_TEST_ENABLE__POR,
	[TOMTOM_A_MICB_CFILT_1_CTL] = TOMTOM_A_MICB_CFILT_1_CTL__POR,
	[TOMTOM_A_MICB_CFILT_1_VAL] = TOMTOM_A_MICB_CFILT_1_VAL__POR,
	[TOMTOM_A_MICB_CFILT_1_PRECHRG] = TOMTOM_A_MICB_CFILT_1_PRECHRG__POR,
	[TOMTOM_A_MICB_1_CTL] = TOMTOM_A_MICB_1_CTL__POR,
	[TOMTOM_A_MICB_1_INT_RBIAS] = TOMTOM_A_MICB_1_INT_RBIAS__POR,
	[TOMTOM_A_MICB_1_MBHC] = TOMTOM_A_MICB_1_MBHC__POR,
	[TOMTOM_A_MICB_CFILT_2_CTL] = TOMTOM_A_MICB_CFILT_2_CTL__POR,
	[TOMTOM_A_MICB_CFILT_2_VAL] = TOMTOM_A_MICB_CFILT_2_VAL__POR,
	[TOMTOM_A_MICB_CFILT_2_PRECHRG] = TOMTOM_A_MICB_CFILT_2_PRECHRG__POR,
	[TOMTOM_A_MICB_2_CTL] = TOMTOM_A_MICB_2_CTL__POR,
	[TOMTOM_A_MICB_2_INT_RBIAS] = TOMTOM_A_MICB_2_INT_RBIAS__POR,
	[TOMTOM_A_MICB_2_MBHC] = TOMTOM_A_MICB_2_MBHC__POR,
	[TOMTOM_A_MICB_CFILT_3_CTL] = TOMTOM_A_MICB_CFILT_3_CTL__POR,
	[TOMTOM_A_MICB_CFILT_3_VAL] = TOMTOM_A_MICB_CFILT_3_VAL__POR,
	[TOMTOM_A_MICB_CFILT_3_PRECHRG] = TOMTOM_A_MICB_CFILT_3_PRECHRG__POR,
	[TOMTOM_A_MICB_3_CTL] = TOMTOM_A_MICB_3_CTL__POR,
	[TOMTOM_A_MICB_3_INT_RBIAS] = TOMTOM_A_MICB_3_INT_RBIAS__POR,
	[TOMTOM_A_MICB_3_MBHC] = TOMTOM_A_MICB_3_MBHC__POR,
	[TOMTOM_A_MICB_4_CTL] = TOMTOM_A_MICB_4_CTL__POR,
	[TOMTOM_A_MICB_4_INT_RBIAS] = TOMTOM_A_MICB_4_INT_RBIAS__POR,
	[TOMTOM_A_MICB_4_MBHC] = TOMTOM_A_MICB_4_MBHC__POR,
	[TOMTOM_A_SPKR_DRV2_EN] = TOMTOM_A_SPKR_DRV2_EN__POR,
	[TOMTOM_A_SPKR_DRV2_GAIN] = TOMTOM_A_SPKR_DRV2_GAIN__POR,
	[TOMTOM_A_SPKR_DRV2_DAC_CTL] = TOMTOM_A_SPKR_DRV2_DAC_CTL__POR,
	[TOMTOM_A_SPKR_DRV2_OCP_CTL] = TOMTOM_A_SPKR_DRV2_OCP_CTL__POR,
	[TOMTOM_A_SPKR_DRV2_CLIP_DET] = TOMTOM_A_SPKR_DRV2_CLIP_DET__POR,
	[TOMTOM_A_SPKR_DRV2_DBG_DAC] = TOMTOM_A_SPKR_DRV2_DBG_DAC__POR,
	[TOMTOM_A_SPKR_DRV2_DBG_PA] = TOMTOM_A_SPKR_DRV2_DBG_PA__POR,
	[TOMTOM_A_SPKR_DRV2_DBG_PWRSTG] = TOMTOM_A_SPKR_DRV2_DBG_PWRSTG__POR,
	[TOMTOM_A_SPKR_DRV2_BIAS_LDO] = TOMTOM_A_SPKR_DRV2_BIAS_LDO__POR,
	[TOMTOM_A_SPKR_DRV2_BIAS_INT] = TOMTOM_A_SPKR_DRV2_BIAS_INT__POR,
	[TOMTOM_A_SPKR_DRV2_BIAS_PA] = TOMTOM_A_SPKR_DRV2_BIAS_PA__POR,
	[TOMTOM_A_SPKR_DRV2_STATUS_OCP] = TOMTOM_A_SPKR_DRV2_STATUS_OCP__POR,
	[TOMTOM_A_SPKR_DRV2_STATUS_PA] = TOMTOM_A_SPKR_DRV2_STATUS_PA__POR,
	[TOMTOM_A_MBHC_INSERT_DETECT] = TOMTOM_A_MBHC_INSERT_DETECT__POR,
	[TOMTOM_A_MBHC_INSERT_DET_STATUS] =
				TOMTOM_A_MBHC_INSERT_DET_STATUS__POR,
	[TOMTOM_A_TX_COM_BIAS] = TOMTOM_A_TX_COM_BIAS__POR,
	[TOMTOM_A_MBHC_INSERT_DETECT2] = TOMTOM_A_MBHC_INSERT_DETECT2__POR,
	[TOMTOM_A_MBHC_SCALING_MUX_1] = TOMTOM_A_MBHC_SCALING_MUX_1__POR,
	[TOMTOM_A_MBHC_SCALING_MUX_2] = TOMTOM_A_MBHC_SCALING_MUX_2__POR,
	[TOMTOM_A_MAD_ANA_CTRL] = TOMTOM_A_MAD_ANA_CTRL__POR,
	[TOMTOM_A_TX_SUP_SWITCH_CTRL_1] = TOMTOM_A_TX_SUP_SWITCH_CTRL_1__POR,
	[TOMTOM_A_TX_SUP_SWITCH_CTRL_2] = TOMTOM_A_TX_SUP_SWITCH_CTRL_2__POR,
	[TOMTOM_A_TX_1_GAIN] = TOMTOM_A_TX_1_GAIN__POR,
	[TOMTOM_A_TX_1_2_TEST_EN] = TOMTOM_A_TX_1_2_TEST_EN__POR,
	[TOMTOM_A_TX_2_GAIN] = TOMTOM_A_TX_2_GAIN__POR,
	[TOMTOM_A_TX_1_2_ADC_IB] = TOMTOM_A_TX_1_2_ADC_IB__POR,
	[TOMTOM_A_TX_1_2_ATEST_REFCTRL] = TOMTOM_A_TX_1_2_ATEST_REFCTRL__POR,
	[TOMTOM_A_TX_1_2_TEST_CTL] = TOMTOM_A_TX_1_2_TEST_CTL__POR,
	[TOMTOM_A_TX_1_2_TEST_BLOCK_EN] = TOMTOM_A_TX_1_2_TEST_BLOCK_EN__POR,
	[TOMTOM_A_TX_1_2_TXFE_CLKDIV] = TOMTOM_A_TX_1_2_TXFE_CLKDIV__POR,
	[TOMTOM_A_TX_1_2_SAR_ERR_CH1] = TOMTOM_A_TX_1_2_SAR_ERR_CH1__POR,
	[TOMTOM_A_TX_1_2_SAR_ERR_CH2] = TOMTOM_A_TX_1_2_SAR_ERR_CH2__POR,
	[TOMTOM_A_TX_3_GAIN] = TOMTOM_A_TX_3_GAIN__POR,
	[TOMTOM_A_TX_3_4_TEST_EN] = TOMTOM_A_TX_3_4_TEST_EN__POR,
	[TOMTOM_A_TX_4_GAIN] = TOMTOM_A_TX_4_GAIN__POR,
	[TOMTOM_A_TX_3_4_ADC_IB] = TOMTOM_A_TX_3_4_ADC_IB__POR,
	[TOMTOM_A_TX_3_4_ATEST_REFCTRL] = TOMTOM_A_TX_3_4_ATEST_REFCTRL__POR,
	[TOMTOM_A_TX_3_4_TEST_CTL] = TOMTOM_A_TX_3_4_TEST_CTL__POR,
	[TOMTOM_A_TX_3_4_TEST_BLOCK_EN] = TOMTOM_A_TX_3_4_TEST_BLOCK_EN__POR,
	[TOMTOM_A_TX_3_4_TXFE_CKDIV] = TOMTOM_A_TX_3_4_TXFE_CKDIV__POR,
	[TOMTOM_A_TX_3_4_SAR_ERR_CH3] = TOMTOM_A_TX_3_4_SAR_ERR_CH3__POR,
	[TOMTOM_A_TX_3_4_SAR_ERR_CH4] = TOMTOM_A_TX_3_4_SAR_ERR_CH4__POR,
	[TOMTOM_A_TX_5_GAIN] = TOMTOM_A_TX_5_GAIN__POR,
	[TOMTOM_A_TX_5_6_TEST_EN] = TOMTOM_A_TX_5_6_TEST_EN__POR,
	[TOMTOM_A_TX_6_GAIN] = TOMTOM_A_TX_6_GAIN__POR,
	[TOMTOM_A_TX_5_6_ADC_IB] = TOMTOM_A_TX_5_6_ADC_IB__POR,
	[TOMTOM_A_TX_5_6_ATEST_REFCTRL] = TOMTOM_A_TX_5_6_ATEST_REFCTRL__POR,
	[TOMTOM_A_TX_5_6_TEST_CTL] = TOMTOM_A_TX_5_6_TEST_CTL__POR,
	[TOMTOM_A_TX_5_6_TEST_BLOCK_EN] = TOMTOM_A_TX_5_6_TEST_BLOCK_EN__POR,
	[TOMTOM_A_TX_5_6_TXFE_CKDIV] = TOMTOM_A_TX_5_6_TXFE_CKDIV__POR,
	[TOMTOM_A_TX_5_6_SAR_ERR_CH5] = TOMTOM_A_TX_5_6_SAR_ERR_CH5__POR,
	[TOMTOM_A_TX_5_6_SAR_ERR_CH6] = TOMTOM_A_TX_5_6_SAR_ERR_CH6__POR,
	[TOMTOM_A_TX_7_MBHC_EN] = TOMTOM_A_TX_7_MBHC_EN__POR,
	[TOMTOM_A_TX_7_MBHC_ATEST_REFCTRL] =
				TOMTOM_A_TX_7_MBHC_ATEST_REFCTRL__POR,
	[TOMTOM_A_TX_7_MBHC_ADC] = TOMTOM_A_TX_7_MBHC_ADC__POR,
	[TOMTOM_A_TX_7_MBHC_TEST_CTL] = TOMTOM_A_TX_7_MBHC_TEST_CTL__POR,
	[TOMTOM_A_TX_7_MBHC_SAR_ERR] = TOMTOM_A_TX_7_MBHC_SAR_ERR__POR,
	[TOMTOM_A_TX_7_TXFE_CLKDIV] = TOMTOM_A_TX_7_TXFE_CLKDIV__POR,
	[TOMTOM_A_RCO_CTRL] = TOMTOM_A_RCO_CTRL__POR,
	[TOMTOM_A_RCO_CALIBRATION_CTRL1] = TOMTOM_A_RCO_CALIBRATION_CTRL1__POR,
	[TOMTOM_A_RCO_CALIBRATION_CTRL2] = TOMTOM_A_RCO_CALIBRATION_CTRL2__POR,
	[TOMTOM_A_RCO_CALIBRATION_CTRL3] = TOMTOM_A_RCO_CALIBRATION_CTRL3__POR,
	[TOMTOM_A_RCO_TEST_CTRL] = TOMTOM_A_RCO_TEST_CTRL__POR,
	[TOMTOM_A_RCO_CALIBRATION_RESULT1] =
				TOMTOM_A_RCO_CALIBRATION_RESULT1__POR,
	[TOMTOM_A_RCO_CALIBRATION_RESULT2] =
				TOMTOM_A_RCO_CALIBRATION_RESULT2__POR,
	[TOMTOM_A_BUCK_MODE_1] = TOMTOM_A_BUCK_MODE_1__POR,
	[TOMTOM_A_BUCK_MODE_2] = TOMTOM_A_BUCK_MODE_2__POR,
	[TOMTOM_A_BUCK_MODE_3] = TOMTOM_A_BUCK_MODE_3__POR,
	[TOMTOM_A_BUCK_MODE_4] = TOMTOM_A_BUCK_MODE_4__POR,
	[TOMTOM_A_BUCK_MODE_5] = TOMTOM_A_BUCK_MODE_5__POR,
	[TOMTOM_A_BUCK_CTRL_VCL_1] = TOMTOM_A_BUCK_CTRL_VCL_1__POR,
	[TOMTOM_A_BUCK_CTRL_VCL_2] = TOMTOM_A_BUCK_CTRL_VCL_2__POR,
	[TOMTOM_A_BUCK_CTRL_VCL_3] = TOMTOM_A_BUCK_CTRL_VCL_3__POR,
	[TOMTOM_A_BUCK_CTRL_CCL_1] = TOMTOM_A_BUCK_CTRL_CCL_1__POR,
	[TOMTOM_A_BUCK_CTRL_CCL_2] = TOMTOM_A_BUCK_CTRL_CCL_2__POR,
	[TOMTOM_A_BUCK_CTRL_CCL_3] = TOMTOM_A_BUCK_CTRL_CCL_3__POR,
	[TOMTOM_A_BUCK_CTRL_CCL_4] = TOMTOM_A_BUCK_CTRL_CCL_4__POR,
	[TOMTOM_A_BUCK_CTRL_PWM_DRVR_1] = TOMTOM_A_BUCK_CTRL_PWM_DRVR_1__POR,
	[TOMTOM_A_BUCK_CTRL_PWM_DRVR_2] = TOMTOM_A_BUCK_CTRL_PWM_DRVR_2__POR,
	[TOMTOM_A_BUCK_CTRL_PWM_DRVR_3] = TOMTOM_A_BUCK_CTRL_PWM_DRVR_3__POR,
	[TOMTOM_A_BUCK_TMUX_A_D] = TOMTOM_A_BUCK_TMUX_A_D__POR,
	[TOMTOM_A_NCP_BUCKREF] = TOMTOM_A_NCP_BUCKREF__POR,
	[TOMTOM_A_NCP_EN] = TOMTOM_A_NCP_EN__POR,
	[TOMTOM_A_NCP_CLK] = TOMTOM_A_NCP_CLK__POR,
	[TOMTOM_A_NCP_STATIC] = TOMTOM_A_NCP_STATIC__POR,
	[TOMTOM_A_NCP_VTH_LOW] = TOMTOM_A_NCP_VTH_LOW__POR,
	[TOMTOM_A_NCP_VTH_HIGH] = TOMTOM_A_NCP_VTH_HIGH__POR,
	[TOMTOM_A_NCP_ATEST] = TOMTOM_A_NCP_ATEST__POR,
	[TOMTOM_A_NCP_DTEST] = TOMTOM_A_NCP_DTEST__POR,
	[TOMTOM_A_NCP_DLY1] = TOMTOM_A_NCP_DLY1__POR,
	[TOMTOM_A_NCP_DLY2] = TOMTOM_A_NCP_DLY2__POR,
	[TOMTOM_A_RX_AUX_SW_CTL] = TOMTOM_A_RX_AUX_SW_CTL__POR,
	[TOMTOM_A_RX_PA_AUX_IN_CONN] = TOMTOM_A_RX_PA_AUX_IN_CONN__POR,
	[TOMTOM_A_RX_COM_TIMER_DIV] = TOMTOM_A_RX_COM_TIMER_DIV__POR,
	[TOMTOM_A_RX_COM_OCP_CTL] = TOMTOM_A_RX_COM_OCP_CTL__POR,
	[TOMTOM_A_RX_COM_OCP_COUNT] = TOMTOM_A_RX_COM_OCP_COUNT__POR,
	[TOMTOM_A_RX_COM_DAC_CTL] = TOMTOM_A_RX_COM_DAC_CTL__POR,
	[TOMTOM_A_RX_COM_BIAS] = TOMTOM_A_RX_COM_BIAS__POR,
	[TOMTOM_A_RX_HPH_AUTO_CHOP] = TOMTOM_A_RX_HPH_AUTO_CHOP__POR,
	[TOMTOM_A_RX_HPH_CHOP_CTL] = TOMTOM_A_RX_HPH_CHOP_CTL__POR,
	[TOMTOM_A_RX_HPH_BIAS_PA] = TOMTOM_A_RX_HPH_BIAS_PA__POR,
	[TOMTOM_A_RX_HPH_BIAS_LDO] = TOMTOM_A_RX_HPH_BIAS_LDO__POR,
	[TOMTOM_A_RX_HPH_BIAS_CNP] = TOMTOM_A_RX_HPH_BIAS_CNP__POR,
	[TOMTOM_A_RX_HPH_BIAS_WG_OCP] = TOMTOM_A_RX_HPH_BIAS_WG_OCP__POR,
	[TOMTOM_A_RX_HPH_OCP_CTL] = TOMTOM_A_RX_HPH_OCP_CTL__POR,
	[TOMTOM_A_RX_HPH_CNP_EN] = TOMTOM_A_RX_HPH_CNP_EN__POR,
	[TOMTOM_A_RX_HPH_CNP_WG_CTL] = TOMTOM_A_RX_HPH_CNP_WG_CTL__POR,
	[TOMTOM_A_RX_HPH_CNP_WG_TIME] = TOMTOM_A_RX_HPH_CNP_WG_TIME__POR,
	[TOMTOM_A_RX_HPH_L_GAIN] = TOMTOM_A_RX_HPH_L_GAIN__POR,
	[TOMTOM_A_RX_HPH_L_TEST] = TOMTOM_A_RX_HPH_L_TEST__POR,
	[TOMTOM_A_RX_HPH_L_PA_CTL] = TOMTOM_A_RX_HPH_L_PA_CTL__POR,
	[TOMTOM_A_RX_HPH_L_DAC_CTL] = TOMTOM_A_RX_HPH_L_DAC_CTL__POR,
	[TOMTOM_A_RX_HPH_L_ATEST] = TOMTOM_A_RX_HPH_L_ATEST__POR,
	[TOMTOM_A_RX_HPH_L_STATUS] = TOMTOM_A_RX_HPH_L_STATUS__POR,
	[TOMTOM_A_RX_HPH_R_GAIN] = TOMTOM_A_RX_HPH_R_GAIN__POR,
	[TOMTOM_A_RX_HPH_R_TEST] = TOMTOM_A_RX_HPH_R_TEST__POR,
	[TOMTOM_A_RX_HPH_R_PA_CTL] = TOMTOM_A_RX_HPH_R_PA_CTL__POR,
	[TOMTOM_A_RX_HPH_R_DAC_CTL] = TOMTOM_A_RX_HPH_R_DAC_CTL__POR,
	[TOMTOM_A_RX_HPH_R_ATEST] = TOMTOM_A_RX_HPH_R_ATEST__POR,
	[TOMTOM_A_RX_HPH_R_STATUS] = TOMTOM_A_RX_HPH_R_STATUS__POR,
	[TOMTOM_A_RX_EAR_BIAS_PA] = TOMTOM_A_RX_EAR_BIAS_PA__POR,
	[TOMTOM_A_RX_EAR_BIAS_CMBUFF] = TOMTOM_A_RX_EAR_BIAS_CMBUFF__POR,
	[TOMTOM_A_RX_EAR_EN] = TOMTOM_A_RX_EAR_EN__POR,
	[TOMTOM_A_RX_EAR_GAIN] = TOMTOM_A_RX_EAR_GAIN__POR,
	[TOMTOM_A_RX_EAR_CMBUFF] = TOMTOM_A_RX_EAR_CMBUFF__POR,
	[TOMTOM_A_RX_EAR_ICTL] = TOMTOM_A_RX_EAR_ICTL__POR,
	[TOMTOM_A_RX_EAR_CCOMP] = TOMTOM_A_RX_EAR_CCOMP__POR,
	[TOMTOM_A_RX_EAR_VCM] = TOMTOM_A_RX_EAR_VCM__POR,
	[TOMTOM_A_RX_EAR_CNP] = TOMTOM_A_RX_EAR_CNP__POR,
	[TOMTOM_A_RX_EAR_DAC_CTL_ATEST] = TOMTOM_A_RX_EAR_DAC_CTL_ATEST__POR,
	[TOMTOM_A_RX_EAR_STATUS] = TOMTOM_A_RX_EAR_STATUS__POR,
	[TOMTOM_A_RX_LINE_BIAS_PA] = TOMTOM_A_RX_LINE_BIAS_PA__POR,
	[TOMTOM_A_RX_BUCK_BIAS1] = TOMTOM_A_RX_BUCK_BIAS1__POR,
	[TOMTOM_A_RX_BUCK_BIAS2] = TOMTOM_A_RX_BUCK_BIAS2__POR,
	[TOMTOM_A_RX_LINE_COM] = TOMTOM_A_RX_LINE_COM__POR,
	[TOMTOM_A_RX_LINE_CNP_EN] = TOMTOM_A_RX_LINE_CNP_EN__POR,
	[TOMTOM_A_RX_LINE_CNP_WG_CTL] = TOMTOM_A_RX_LINE_CNP_WG_CTL__POR,
	[TOMTOM_A_RX_LINE_CNP_WG_TIME] = TOMTOM_A_RX_LINE_CNP_WG_TIME__POR,
	[TOMTOM_A_RX_LINE_1_GAIN] = TOMTOM_A_RX_LINE_1_GAIN__POR,
	[TOMTOM_A_RX_LINE_1_TEST] = TOMTOM_A_RX_LINE_1_TEST__POR,
	[TOMTOM_A_RX_LINE_1_DAC_CTL] = TOMTOM_A_RX_LINE_1_DAC_CTL__POR,
	[TOMTOM_A_RX_LINE_1_STATUS] = TOMTOM_A_RX_LINE_1_STATUS__POR,
	[TOMTOM_A_RX_LINE_2_GAIN] = TOMTOM_A_RX_LINE_2_GAIN__POR,
	[TOMTOM_A_RX_LINE_2_TEST] = TOMTOM_A_RX_LINE_2_TEST__POR,
	[TOMTOM_A_RX_LINE_2_DAC_CTL] = TOMTOM_A_RX_LINE_2_DAC_CTL__POR,
	[TOMTOM_A_RX_LINE_2_STATUS] = TOMTOM_A_RX_LINE_2_STATUS__POR,
	[TOMTOM_A_RX_LINE_3_GAIN] = TOMTOM_A_RX_LINE_3_GAIN__POR,
	[TOMTOM_A_RX_LINE_3_TEST] = TOMTOM_A_RX_LINE_3_TEST__POR,
	[TOMTOM_A_RX_LINE_3_DAC_CTL] = TOMTOM_A_RX_LINE_3_DAC_CTL__POR,
	[TOMTOM_A_RX_LINE_3_STATUS] = TOMTOM_A_RX_LINE_3_STATUS__POR,
	[TOMTOM_A_RX_LINE_4_GAIN] = TOMTOM_A_RX_LINE_4_GAIN__POR,
	[TOMTOM_A_RX_LINE_4_TEST] = TOMTOM_A_RX_LINE_4_TEST__POR,
	[TOMTOM_A_RX_LINE_4_DAC_CTL] = TOMTOM_A_RX_LINE_4_DAC_CTL__POR,
	[TOMTOM_A_RX_LINE_4_STATUS] = TOMTOM_A_RX_LINE_4_STATUS__POR,
	[TOMTOM_A_RX_LINE_CNP_DBG] = TOMTOM_A_RX_LINE_CNP_DBG__POR,
	[TOMTOM_A_SPKR_DRV1_EN] = TOMTOM_A_SPKR_DRV1_EN__POR,
	[TOMTOM_A_SPKR_DRV1_GAIN] = TOMTOM_A_SPKR_DRV1_GAIN__POR,
	[TOMTOM_A_SPKR_DRV1_DAC_CTL] = TOMTOM_A_SPKR_DRV1_DAC_CTL__POR,
	[TOMTOM_A_SPKR_DRV1_OCP_CTL] = TOMTOM_A_SPKR_DRV1_OCP_CTL__POR,
	[TOMTOM_A_SPKR_DRV1_CLIP_DET] = TOMTOM_A_SPKR_DRV1_CLIP_DET__POR,
	[TOMTOM_A_SPKR_DRV1_IEC] = TOMTOM_A_SPKR_DRV1_IEC__POR,
	[TOMTOM_A_SPKR_DRV1_DBG_DAC] = TOMTOM_A_SPKR_DRV1_DBG_DAC__POR,
	[TOMTOM_A_SPKR_DRV1_DBG_PA] = TOMTOM_A_SPKR_DRV1_DBG_PA__POR,
	[TOMTOM_A_SPKR_DRV1_DBG_PWRSTG] = TOMTOM_A_SPKR_DRV1_DBG_PWRSTG__POR,
	[TOMTOM_A_SPKR_DRV1_BIAS_LDO] = TOMTOM_A_SPKR_DRV1_BIAS_LDO__POR,
	[TOMTOM_A_SPKR_DRV1_BIAS_INT] = TOMTOM_A_SPKR_DRV1_BIAS_INT__POR,
	[TOMTOM_A_SPKR_DRV1_BIAS_PA] = TOMTOM_A_SPKR_DRV1_BIAS_PA__POR,
	[TOMTOM_A_SPKR_DRV1_STATUS_OCP] = TOMTOM_A_SPKR_DRV1_STATUS_OCP__POR,
	[TOMTOM_A_SPKR_DRV1_STATUS_PA] = TOMTOM_A_SPKR_DRV1_STATUS_PA__POR,
	[TOMTOM_A_SPKR1_PROT_EN] = TOMTOM_A_SPKR1_PROT_EN__POR,
	[TOMTOM_A_SPKR1_PROT_ADC_TEST_EN] =
				TOMTOM_A_SPKR1_PROT_ADC_TEST_EN__POR,
	[TOMTOM_A_SPKR1_PROT_ATEST] = TOMTOM_A_SPKR1_PROT_ATEST__POR,
	[TOMTOM_A_SPKR1_PROT_LDO_CTRL] = TOMTOM_A_SPKR1_PROT_LDO_CTRL__POR,
	[TOMTOM_A_SPKR1_PROT_ISENSE_CTRL] =
				TOMTOM_A_SPKR1_PROT_ISENSE_CTRL__POR,
	[TOMTOM_A_SPKR1_PROT_VSENSE_CTRL] =
				TOMTOM_A_SPKR1_PROT_VSENSE_CTRL__POR,
	[TOMTOM_A_SPKR2_PROT_EN] = TOMTOM_A_SPKR2_PROT_EN__POR,
	[TOMTOM_A_SPKR2_PROT_ADC_TEST_EN] =
				TOMTOM_A_SPKR2_PROT_ADC_TEST_EN__POR,
	[TOMTOM_A_SPKR2_PROT_ATEST] = TOMTOM_A_SPKR2_PROT_ATEST__POR,
	[TOMTOM_A_SPKR2_PROT_LDO_CTRL] = TOMTOM_A_SPKR2_PROT_LDO_CTRL__POR,
	[TOMTOM_A_SPKR2_PROT_ISENSE_CTRL] =
				TOMTOM_A_SPKR2_PROT_ISENSE_CTRL__POR,
	[TOMTOM_A_SPKR2_PROT_VSENSE_CTRL] =
				TOMTOM_A_SPKR2_PROT_VSENSE_CTRL__POR,
	[TOMTOM_A_MBHC_HPH] = TOMTOM_A_MBHC_HPH__POR,
	[TOMTOM_A_CDC_ANC1_B1_CTL] = TOMTOM_A_CDC_ANC1_B1_CTL__POR,
	[TOMTOM_A_CDC_ANC2_B1_CTL] = TOMTOM_A_CDC_ANC2_B1_CTL__POR,
	[TOMTOM_A_CDC_ANC1_SHIFT] = TOMTOM_A_CDC_ANC1_SHIFT__POR,
	[TOMTOM_A_CDC_ANC2_SHIFT] = TOMTOM_A_CDC_ANC2_SHIFT__POR,
	[TOMTOM_A_CDC_ANC1_IIR_B1_CTL] = TOMTOM_A_CDC_ANC1_IIR_B1_CTL__POR,
	[TOMTOM_A_CDC_ANC2_IIR_B1_CTL] = TOMTOM_A_CDC_ANC2_IIR_B1_CTL__POR,
	[TOMTOM_A_CDC_ANC1_IIR_B2_CTL] = TOMTOM_A_CDC_ANC1_IIR_B2_CTL__POR,
	[TOMTOM_A_CDC_ANC2_IIR_B2_CTL] = TOMTOM_A_CDC_ANC2_IIR_B2_CTL__POR,
	[TOMTOM_A_CDC_ANC1_IIR_B3_CTL] = TOMTOM_A_CDC_ANC1_IIR_B3_CTL__POR,
	[TOMTOM_A_CDC_ANC2_IIR_B3_CTL] = TOMTOM_A_CDC_ANC2_IIR_B3_CTL__POR,
	[TOMTOM_A_CDC_ANC1_LPF_B1_CTL] = TOMTOM_A_CDC_ANC1_LPF_B1_CTL__POR,
	[TOMTOM_A_CDC_ANC2_LPF_B1_CTL] = TOMTOM_A_CDC_ANC2_LPF_B1_CTL__POR,
	[TOMTOM_A_CDC_ANC1_LPF_B2_CTL] = TOMTOM_A_CDC_ANC1_LPF_B2_CTL__POR,
	[TOMTOM_A_CDC_ANC2_LPF_B2_CTL] = TOMTOM_A_CDC_ANC2_LPF_B2_CTL__POR,
	[TOMTOM_A_CDC_ANC1_SPARE] = TOMTOM_A_CDC_ANC1_SPARE__POR,
	[TOMTOM_A_CDC_ANC2_SPARE] = TOMTOM_A_CDC_ANC2_SPARE__POR,
	[TOMTOM_A_CDC_ANC1_SMLPF_CTL] = TOMTOM_A_CDC_ANC1_SMLPF_CTL__POR,
	[TOMTOM_A_CDC_ANC2_SMLPF_CTL] = TOMTOM_A_CDC_ANC2_SMLPF_CTL__POR,
	[TOMTOM_A_CDC_ANC1_DCFLT_CTL] = TOMTOM_A_CDC_ANC1_DCFLT_CTL__POR,
	[TOMTOM_A_CDC_ANC2_DCFLT_CTL] = TOMTOM_A_CDC_ANC2_DCFLT_CTL__POR,
	[TOMTOM_A_CDC_ANC1_GAIN_CTL] = TOMTOM_A_CDC_ANC1_GAIN_CTL__POR,
	[TOMTOM_A_CDC_ANC2_GAIN_CTL] = TOMTOM_A_CDC_ANC2_GAIN_CTL__POR,
	[TOMTOM_A_CDC_ANC1_B2_CTL] = TOMTOM_A_CDC_ANC1_B2_CTL__POR,
	[TOMTOM_A_CDC_ANC2_B2_CTL] = TOMTOM_A_CDC_ANC2_B2_CTL__POR,
	[TOMTOM_A_CDC_TX1_VOL_CTL_TIMER] = TOMTOM_A_CDC_TX1_VOL_CTL_TIMER__POR,
	[TOMTOM_A_CDC_TX2_VOL_CTL_TIMER] = TOMTOM_A_CDC_TX2_VOL_CTL_TIMER__POR,
	[TOMTOM_A_CDC_TX3_VOL_CTL_TIMER] = TOMTOM_A_CDC_TX3_VOL_CTL_TIMER__POR,
	[TOMTOM_A_CDC_TX4_VOL_CTL_TIMER] = TOMTOM_A_CDC_TX4_VOL_CTL_TIMER__POR,
	[TOMTOM_A_CDC_TX5_VOL_CTL_TIMER] = TOMTOM_A_CDC_TX5_VOL_CTL_TIMER__POR,
	[TOMTOM_A_CDC_TX6_VOL_CTL_TIMER] = TOMTOM_A_CDC_TX6_VOL_CTL_TIMER__POR,
	[TOMTOM_A_CDC_TX7_VOL_CTL_TIMER] = TOMTOM_A_CDC_TX7_VOL_CTL_TIMER__POR,
	[TOMTOM_A_CDC_TX8_VOL_CTL_TIMER] = TOMTOM_A_CDC_TX8_VOL_CTL_TIMER__POR,
	[TOMTOM_A_CDC_TX9_VOL_CTL_TIMER] = TOMTOM_A_CDC_TX9_VOL_CTL_TIMER__POR,
	[TOMTOM_A_CDC_TX10_VOL_CTL_TIMER] =
				TOMTOM_A_CDC_TX10_VOL_CTL_TIMER__POR,
	[TOMTOM_A_CDC_TX1_VOL_CTL_GAIN] = TOMTOM_A_CDC_TX1_VOL_CTL_GAIN__POR,
	[TOMTOM_A_CDC_TX2_VOL_CTL_GAIN] = TOMTOM_A_CDC_TX2_VOL_CTL_GAIN__POR,
	[TOMTOM_A_CDC_TX3_VOL_CTL_GAIN] = TOMTOM_A_CDC_TX3_VOL_CTL_GAIN__POR,
	[TOMTOM_A_CDC_TX4_VOL_CTL_GAIN] = TOMTOM_A_CDC_TX4_VOL_CTL_GAIN__POR,
	[TOMTOM_A_CDC_TX5_VOL_CTL_GAIN] = TOMTOM_A_CDC_TX5_VOL_CTL_GAIN__POR,
	[TOMTOM_A_CDC_TX6_VOL_CTL_GAIN] = TOMTOM_A_CDC_TX6_VOL_CTL_GAIN__POR,
	[TOMTOM_A_CDC_TX7_VOL_CTL_GAIN] = TOMTOM_A_CDC_TX7_VOL_CTL_GAIN__POR,
	[TOMTOM_A_CDC_TX8_VOL_CTL_GAIN] = TOMTOM_A_CDC_TX8_VOL_CTL_GAIN__POR,
	[TOMTOM_A_CDC_TX9_VOL_CTL_GAIN] = TOMTOM_A_CDC_TX9_VOL_CTL_GAIN__POR,
	[TOMTOM_A_CDC_TX10_VOL_CTL_GAIN] = TOMTOM_A_CDC_TX10_VOL_CTL_GAIN__POR,
	[TOMTOM_A_CDC_TX1_VOL_CTL_CFG] = TOMTOM_A_CDC_TX1_VOL_CTL_CFG__POR,
	[TOMTOM_A_CDC_TX2_VOL_CTL_CFG] = TOMTOM_A_CDC_TX2_VOL_CTL_CFG__POR,
	[TOMTOM_A_CDC_TX3_VOL_CTL_CFG] = TOMTOM_A_CDC_TX3_VOL_CTL_CFG__POR,
	[TOMTOM_A_CDC_TX4_VOL_CTL_CFG] = TOMTOM_A_CDC_TX4_VOL_CTL_CFG__POR,
	[TOMTOM_A_CDC_TX5_VOL_CTL_CFG] = TOMTOM_A_CDC_TX5_VOL_CTL_CFG__POR,
	[TOMTOM_A_CDC_TX6_VOL_CTL_CFG] = TOMTOM_A_CDC_TX6_VOL_CTL_CFG__POR,
	[TOMTOM_A_CDC_TX7_VOL_CTL_CFG] = TOMTOM_A_CDC_TX7_VOL_CTL_CFG__POR,
	[TOMTOM_A_CDC_TX8_VOL_CTL_CFG] = TOMTOM_A_CDC_TX8_VOL_CTL_CFG__POR,
	[TOMTOM_A_CDC_TX9_VOL_CTL_CFG] = TOMTOM_A_CDC_TX9_VOL_CTL_CFG__POR,
	[TOMTOM_A_CDC_TX10_VOL_CTL_CFG] = TOMTOM_A_CDC_TX10_VOL_CTL_CFG__POR,
	[TOMTOM_A_CDC_TX1_MUX_CTL] = TOMTOM_A_CDC_TX1_MUX_CTL__POR,
	[TOMTOM_A_CDC_TX2_MUX_CTL] = TOMTOM_A_CDC_TX2_MUX_CTL__POR,
	[TOMTOM_A_CDC_TX3_MUX_CTL] = TOMTOM_A_CDC_TX3_MUX_CTL__POR,
	[TOMTOM_A_CDC_TX4_MUX_CTL] = TOMTOM_A_CDC_TX4_MUX_CTL__POR,
	[TOMTOM_A_CDC_TX5_MUX_CTL] = TOMTOM_A_CDC_TX5_MUX_CTL__POR,
	[TOMTOM_A_CDC_TX6_MUX_CTL] = TOMTOM_A_CDC_TX6_MUX_CTL__POR,
	[TOMTOM_A_CDC_TX7_MUX_CTL] = TOMTOM_A_CDC_TX7_MUX_CTL__POR,
	[TOMTOM_A_CDC_TX8_MUX_CTL] = TOMTOM_A_CDC_TX8_MUX_CTL__POR,
	[TOMTOM_A_CDC_TX9_MUX_CTL] = TOMTOM_A_CDC_TX9_MUX_CTL__POR,
	[TOMTOM_A_CDC_TX10_MUX_CTL] = TOMTOM_A_CDC_TX10_MUX_CTL__POR,
	[TOMTOM_A_CDC_TX1_CLK_FS_CTL] = TOMTOM_A_CDC_TX1_CLK_FS_CTL__POR,
	[TOMTOM_A_CDC_TX2_CLK_FS_CTL] = TOMTOM_A_CDC_TX2_CLK_FS_CTL__POR,
	[TOMTOM_A_CDC_TX3_CLK_FS_CTL] = TOMTOM_A_CDC_TX3_CLK_FS_CTL__POR,
	[TOMTOM_A_CDC_TX4_CLK_FS_CTL] = TOMTOM_A_CDC_TX4_CLK_FS_CTL__POR,
	[TOMTOM_A_CDC_TX5_CLK_FS_CTL] = TOMTOM_A_CDC_TX5_CLK_FS_CTL__POR,
	[TOMTOM_A_CDC_TX6_CLK_FS_CTL] = TOMTOM_A_CDC_TX6_CLK_FS_CTL__POR,
	[TOMTOM_A_CDC_TX7_CLK_FS_CTL] = TOMTOM_A_CDC_TX7_CLK_FS_CTL__POR,
	[TOMTOM_A_CDC_TX8_CLK_FS_CTL] = TOMTOM_A_CDC_TX8_CLK_FS_CTL__POR,
	[TOMTOM_A_CDC_TX9_CLK_FS_CTL] = TOMTOM_A_CDC_TX9_CLK_FS_CTL__POR,
	[TOMTOM_A_CDC_TX10_CLK_FS_CTL] = TOMTOM_A_CDC_TX10_CLK_FS_CTL__POR,
	[TOMTOM_A_CDC_TX1_DMIC_CTL] = TOMTOM_A_CDC_TX1_DMIC_CTL__POR,
	[TOMTOM_A_CDC_TX2_DMIC_CTL] = TOMTOM_A_CDC_TX2_DMIC_CTL__POR,
	[TOMTOM_A_CDC_TX3_DMIC_CTL] = TOMTOM_A_CDC_TX3_DMIC_CTL__POR,
	[TOMTOM_A_CDC_TX4_DMIC_CTL] = TOMTOM_A_CDC_TX4_DMIC_CTL__POR,
	[TOMTOM_A_CDC_TX5_DMIC_CTL] = TOMTOM_A_CDC_TX5_DMIC_CTL__POR,
	[TOMTOM_A_CDC_TX6_DMIC_CTL] = TOMTOM_A_CDC_TX6_DMIC_CTL__POR,
	[TOMTOM_A_CDC_TX7_DMIC_CTL] = TOMTOM_A_CDC_TX7_DMIC_CTL__POR,
	[TOMTOM_A_CDC_TX8_DMIC_CTL] = TOMTOM_A_CDC_TX8_DMIC_CTL__POR,
	[TOMTOM_A_CDC_TX9_DMIC_CTL] = TOMTOM_A_CDC_TX9_DMIC_CTL__POR,
	[TOMTOM_A_CDC_TX10_DMIC_CTL] = TOMTOM_A_CDC_TX10_DMIC_CTL__POR,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL0] = TOMTOM_A_CDC_SPKR_CLIPDET_VAL0__POR,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL1] = TOMTOM_A_CDC_SPKR_CLIPDET_VAL1__POR,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL2] = TOMTOM_A_CDC_SPKR_CLIPDET_VAL2__POR,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL3] = TOMTOM_A_CDC_SPKR_CLIPDET_VAL3__POR,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL4] = TOMTOM_A_CDC_SPKR_CLIPDET_VAL4__POR,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL5] = TOMTOM_A_CDC_SPKR_CLIPDET_VAL5__POR,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL6] = TOMTOM_A_CDC_SPKR_CLIPDET_VAL6__POR,
	[TOMTOM_A_CDC_SPKR_CLIPDET_VAL7] = TOMTOM_A_CDC_SPKR_CLIPDET_VAL7__POR,
	[TOMTOM_A_CDC_DEBUG_B1_CTL] = TOMTOM_A_CDC_DEBUG_B1_CTL__POR,
	[TOMTOM_A_CDC_DEBUG_B2_CTL] = TOMTOM_A_CDC_DEBUG_B2_CTL__POR,
	[TOMTOM_A_CDC_DEBUG_B3_CTL] = TOMTOM_A_CDC_DEBUG_B3_CTL__POR,
	[TOMTOM_A_CDC_DEBUG_B4_CTL] = TOMTOM_A_CDC_DEBUG_B4_CTL__POR,
	[TOMTOM_A_CDC_DEBUG_B5_CTL] = TOMTOM_A_CDC_DEBUG_B5_CTL__POR,
	[TOMTOM_A_CDC_DEBUG_B6_CTL] = TOMTOM_A_CDC_DEBUG_B6_CTL__POR,
	[TOMTOM_A_CDC_DEBUG_B7_CTL] = TOMTOM_A_CDC_DEBUG_B7_CTL__POR,
	[TOMTOM_A_CDC_SRC1_PDA_CFG] = TOMTOM_A_CDC_SRC1_PDA_CFG__POR,
	[TOMTOM_A_CDC_SRC2_PDA_CFG] = TOMTOM_A_CDC_SRC2_PDA_CFG__POR,
	[TOMTOM_A_CDC_SRC1_FS_CTL] = TOMTOM_A_CDC_SRC1_FS_CTL__POR,
	[TOMTOM_A_CDC_SRC2_FS_CTL] = TOMTOM_A_CDC_SRC2_FS_CTL__POR,
	[TOMTOM_A_CDC_RX1_B1_CTL] = TOMTOM_A_CDC_RX1_B1_CTL__POR,
	[TOMTOM_A_CDC_RX2_B1_CTL] = TOMTOM_A_CDC_RX2_B1_CTL__POR,
	[TOMTOM_A_CDC_RX3_B1_CTL] = TOMTOM_A_CDC_RX3_B1_CTL__POR,
	[TOMTOM_A_CDC_RX4_B1_CTL] = TOMTOM_A_CDC_RX4_B1_CTL__POR,
	[TOMTOM_A_CDC_RX5_B1_CTL] = TOMTOM_A_CDC_RX5_B1_CTL__POR,
	[TOMTOM_A_CDC_RX6_B1_CTL] = TOMTOM_A_CDC_RX6_B1_CTL__POR,
	[TOMTOM_A_CDC_RX7_B1_CTL] = TOMTOM_A_CDC_RX7_B1_CTL__POR,
	[TOMTOM_A_CDC_RX1_B2_CTL] = TOMTOM_A_CDC_RX1_B2_CTL__POR,
	[TOMTOM_A_CDC_RX2_B2_CTL] = TOMTOM_A_CDC_RX2_B2_CTL__POR,
	[TOMTOM_A_CDC_RX3_B2_CTL] = TOMTOM_A_CDC_RX3_B2_CTL__POR,
	[TOMTOM_A_CDC_RX4_B2_CTL] = TOMTOM_A_CDC_RX4_B2_CTL__POR,
	[TOMTOM_A_CDC_RX5_B2_CTL] = TOMTOM_A_CDC_RX5_B2_CTL__POR,
	[TOMTOM_A_CDC_RX6_B2_CTL] = TOMTOM_A_CDC_RX6_B2_CTL__POR,
	[TOMTOM_A_CDC_RX7_B2_CTL] = TOMTOM_A_CDC_RX7_B2_CTL__POR,
	[TOMTOM_A_CDC_RX1_B3_CTL] = TOMTOM_A_CDC_RX1_B3_CTL__POR,
	[TOMTOM_A_CDC_RX2_B3_CTL] = TOMTOM_A_CDC_RX2_B3_CTL__POR,
	[TOMTOM_A_CDC_RX3_B3_CTL] = TOMTOM_A_CDC_RX3_B3_CTL__POR,
	[TOMTOM_A_CDC_RX4_B3_CTL] = TOMTOM_A_CDC_RX4_B3_CTL__POR,
	[TOMTOM_A_CDC_RX5_B3_CTL] = TOMTOM_A_CDC_RX5_B3_CTL__POR,
	[TOMTOM_A_CDC_RX6_B3_CTL] = TOMTOM_A_CDC_RX6_B3_CTL__POR,
	[TOMTOM_A_CDC_RX7_B3_CTL] = TOMTOM_A_CDC_RX7_B3_CTL__POR,
	[TOMTOM_A_CDC_RX1_B4_CTL] = TOMTOM_A_CDC_RX1_B4_CTL__POR,
	[TOMTOM_A_CDC_RX2_B4_CTL] = TOMTOM_A_CDC_RX2_B4_CTL__POR,
	[TOMTOM_A_CDC_RX3_B4_CTL] = TOMTOM_A_CDC_RX3_B4_CTL__POR,
	[TOMTOM_A_CDC_RX4_B4_CTL] = TOMTOM_A_CDC_RX4_B4_CTL__POR,
	[TOMTOM_A_CDC_RX5_B4_CTL] = TOMTOM_A_CDC_RX5_B4_CTL__POR,
	[TOMTOM_A_CDC_RX6_B4_CTL] = TOMTOM_A_CDC_RX6_B4_CTL__POR,
	[TOMTOM_A_CDC_RX7_B4_CTL] = TOMTOM_A_CDC_RX7_B4_CTL__POR,
	[TOMTOM_A_CDC_RX1_B5_CTL] = TOMTOM_A_CDC_RX1_B5_CTL__POR,
	[TOMTOM_A_CDC_RX2_B5_CTL] = TOMTOM_A_CDC_RX2_B5_CTL__POR,
	[TOMTOM_A_CDC_RX3_B5_CTL] = TOMTOM_A_CDC_RX3_B5_CTL__POR,
	[TOMTOM_A_CDC_RX4_B5_CTL] = TOMTOM_A_CDC_RX4_B5_CTL__POR,
	[TOMTOM_A_CDC_RX5_B5_CTL] = TOMTOM_A_CDC_RX5_B5_CTL__POR,
	[TOMTOM_A_CDC_RX6_B5_CTL] = TOMTOM_A_CDC_RX6_B5_CTL__POR,
	[TOMTOM_A_CDC_RX7_B5_CTL] = TOMTOM_A_CDC_RX7_B5_CTL__POR,
	[TOMTOM_A_CDC_RX1_B6_CTL] = TOMTOM_A_CDC_RX1_B6_CTL__POR,
	[TOMTOM_A_CDC_RX2_B6_CTL] = TOMTOM_A_CDC_RX2_B6_CTL__POR,
	[TOMTOM_A_CDC_RX3_B6_CTL] = TOMTOM_A_CDC_RX3_B6_CTL__POR,
	[TOMTOM_A_CDC_RX4_B6_CTL] = TOMTOM_A_CDC_RX4_B6_CTL__POR,
	[TOMTOM_A_CDC_RX5_B6_CTL] = TOMTOM_A_CDC_RX5_B6_CTL__POR,
	[TOMTOM_A_CDC_RX6_B6_CTL] = TOMTOM_A_CDC_RX6_B6_CTL__POR,
	[TOMTOM_A_CDC_RX7_B6_CTL] = TOMTOM_A_CDC_RX7_B6_CTL__POR,
	[TOMTOM_A_CDC_RX1_VOL_CTL_B1_CTL] =
				TOMTOM_A_CDC_RX1_VOL_CTL_B1_CTL__POR,
	[TOMTOM_A_CDC_RX2_VOL_CTL_B1_CTL] =
				TOMTOM_A_CDC_RX2_VOL_CTL_B1_CTL__POR,
	[TOMTOM_A_CDC_RX3_VOL_CTL_B1_CTL] =
				TOMTOM_A_CDC_RX3_VOL_CTL_B1_CTL__POR,
	[TOMTOM_A_CDC_RX4_VOL_CTL_B1_CTL] =
				TOMTOM_A_CDC_RX4_VOL_CTL_B1_CTL__POR,
	[TOMTOM_A_CDC_RX5_VOL_CTL_B1_CTL] =
				TOMTOM_A_CDC_RX5_VOL_CTL_B1_CTL__POR,
	[TOMTOM_A_CDC_RX6_VOL_CTL_B1_CTL] =
				TOMTOM_A_CDC_RX6_VOL_CTL_B1_CTL__POR,
	[TOMTOM_A_CDC_RX7_VOL_CTL_B1_CTL] =
				TOMTOM_A_CDC_RX7_VOL_CTL_B1_CTL__POR,
	[TOMTOM_A_CDC_RX1_VOL_CTL_B2_CTL] =
				TOMTOM_A_CDC_RX1_VOL_CTL_B2_CTL__POR,
	[TOMTOM_A_CDC_RX2_VOL_CTL_B2_CTL] =
				TOMTOM_A_CDC_RX2_VOL_CTL_B2_CTL__POR,
	[TOMTOM_A_CDC_RX3_VOL_CTL_B2_CTL] =
				TOMTOM_A_CDC_RX3_VOL_CTL_B2_CTL__POR,
	[TOMTOM_A_CDC_RX4_VOL_CTL_B2_CTL] =
				TOMTOM_A_CDC_RX4_VOL_CTL_B2_CTL__POR,
	[TOMTOM_A_CDC_RX5_VOL_CTL_B2_CTL] =
				TOMTOM_A_CDC_RX5_VOL_CTL_B2_CTL__POR,
	[TOMTOM_A_CDC_RX6_VOL_CTL_B2_CTL] =
				TOMTOM_A_CDC_RX6_VOL_CTL_B2_CTL__POR,
	[TOMTOM_A_CDC_RX7_VOL_CTL_B2_CTL] =
				TOMTOM_A_CDC_RX7_VOL_CTL_B2_CTL__POR,
	[TOMTOM_A_CDC_VBAT_CFG] = TOMTOM_A_CDC_VBAT_CFG__POR,
	[TOMTOM_A_CDC_VBAT_ADC_CAL1] = TOMTOM_A_CDC_VBAT_ADC_CAL1__POR,
	[TOMTOM_A_CDC_VBAT_ADC_CAL2] = TOMTOM_A_CDC_VBAT_ADC_CAL2__POR,
	[TOMTOM_A_CDC_VBAT_ADC_CAL3] = TOMTOM_A_CDC_VBAT_ADC_CAL3__POR,
	[TOMTOM_A_CDC_VBAT_PK_EST1] = TOMTOM_A_CDC_VBAT_PK_EST1__POR,
	[TOMTOM_A_CDC_VBAT_PK_EST2] = TOMTOM_A_CDC_VBAT_PK_EST2__POR,
	[TOMTOM_A_CDC_VBAT_PK_EST3] = TOMTOM_A_CDC_VBAT_PK_EST3__POR,
	[TOMTOM_A_CDC_VBAT_RF_PROC1] = TOMTOM_A_CDC_VBAT_RF_PROC1__POR,
	[TOMTOM_A_CDC_VBAT_RF_PROC2] = TOMTOM_A_CDC_VBAT_RF_PROC2__POR,
	[TOMTOM_A_CDC_VBAT_TAC1] = TOMTOM_A_CDC_VBAT_TAC1__POR,
	[TOMTOM_A_CDC_VBAT_TAC2] = TOMTOM_A_CDC_VBAT_TAC2__POR,
	[TOMTOM_A_CDC_VBAT_TAC3] = TOMTOM_A_CDC_VBAT_TAC3__POR,
	[TOMTOM_A_CDC_VBAT_TAC4] = TOMTOM_A_CDC_VBAT_TAC4__POR,
	[TOMTOM_A_CDC_VBAT_GAIN_UPD1] = TOMTOM_A_CDC_VBAT_GAIN_UPD1__POR,
	[TOMTOM_A_CDC_VBAT_GAIN_UPD2] = TOMTOM_A_CDC_VBAT_GAIN_UPD2__POR,
	[TOMTOM_A_CDC_VBAT_GAIN_UPD3] = TOMTOM_A_CDC_VBAT_GAIN_UPD3__POR,
	[TOMTOM_A_CDC_VBAT_GAIN_UPD4] = TOMTOM_A_CDC_VBAT_GAIN_UPD4__POR,
	[TOMTOM_A_CDC_VBAT_DEBUG1] = TOMTOM_A_CDC_VBAT_DEBUG1__POR,
	[TOMTOM_A_CDC_VBAT_GAIN_UPD_MON] = TOMTOM_A_CDC_VBAT_GAIN_UPD_MON__POR,
	[TOMTOM_A_CDC_VBAT_GAIN_MON_VAL] = TOMTOM_A_CDC_VBAT_GAIN_MON_VAL__POR,
	[TOMTOM_A_CDC_CLK_ANC_RESET_CTL] = TOMTOM_A_CDC_CLK_ANC_RESET_CTL__POR,
	[TOMTOM_A_CDC_CLK_RX_RESET_CTL] = TOMTOM_A_CDC_CLK_RX_RESET_CTL__POR,
	[TOMTOM_A_CDC_CLK_TX_RESET_B1_CTL] =
				TOMTOM_A_CDC_CLK_TX_RESET_B1_CTL__POR,
	[TOMTOM_A_CDC_CLK_TX_RESET_B2_CTL] =
				TOMTOM_A_CDC_CLK_TX_RESET_B2_CTL__POR,
	[TOMTOM_A_CDC_CLK_RX_I2S_CTL] = TOMTOM_A_CDC_CLK_RX_I2S_CTL__POR,
	[TOMTOM_A_CDC_CLK_TX_I2S_CTL] = TOMTOM_A_CDC_CLK_TX_I2S_CTL__POR,
	[TOMTOM_A_CDC_CLK_OTHR_RESET_B1_CTL] =
				TOMTOM_A_CDC_CLK_OTHR_RESET_B1_CTL__POR,
	[TOMTOM_A_CDC_CLK_OTHR_RESET_B2_CTL] =
				TOMTOM_A_CDC_CLK_OTHR_RESET_B2_CTL__POR,
	[TOMTOM_A_CDC_CLK_TX_CLK_EN_B1_CTL] =
				TOMTOM_A_CDC_CLK_TX_CLK_EN_B1_CTL__POR,
	[TOMTOM_A_CDC_CLK_TX_CLK_EN_B2_CTL] =
				TOMTOM_A_CDC_CLK_TX_CLK_EN_B2_CTL__POR,
	[TOMTOM_A_CDC_CLK_OTHR_CTL] = TOMTOM_A_CDC_CLK_OTHR_CTL__POR,
	[TOMTOM_A_CDC_CLK_ANC_CLK_EN_CTL] =
				TOMTOM_A_CDC_CLK_ANC_CLK_EN_CTL__POR,
	[TOMTOM_A_CDC_CLK_RX_B1_CTL] = TOMTOM_A_CDC_CLK_RX_B1_CTL__POR,
	[TOMTOM_A_CDC_CLK_RX_B2_CTL] = TOMTOM_A_CDC_CLK_RX_B2_CTL__POR,
	[TOMTOM_A_CDC_CLK_MCLK_CTL] = TOMTOM_A_CDC_CLK_MCLK_CTL__POR,
	[TOMTOM_A_CDC_CLK_PDM_CTL] = TOMTOM_A_CDC_CLK_PDM_CTL__POR,
	[TOMTOM_A_CDC_CLK_SD_CTL] = TOMTOM_A_CDC_CLK_SD_CTL__POR,
	[TOMTOM_A_CDC_CLSH_B1_CTL] = TOMTOM_A_CDC_CLSH_B1_CTL__POR,
	[TOMTOM_A_CDC_CLSH_B2_CTL] = TOMTOM_A_CDC_CLSH_B2_CTL__POR,
	[TOMTOM_A_CDC_CLSH_B3_CTL] = TOMTOM_A_CDC_CLSH_B3_CTL__POR,
	[TOMTOM_A_CDC_CLSH_BUCK_NCP_VARS] =
				TOMTOM_A_CDC_CLSH_BUCK_NCP_VARS__POR,
	[TOMTOM_A_CDC_CLSH_IDLE_HPH_THSD] =
				TOMTOM_A_CDC_CLSH_IDLE_HPH_THSD__POR,
	[TOMTOM_A_CDC_CLSH_IDLE_EAR_THSD] =
				TOMTOM_A_CDC_CLSH_IDLE_EAR_THSD__POR,
	[TOMTOM_A_CDC_CLSH_FCLKONLY_HPH_THSD] =
				TOMTOM_A_CDC_CLSH_FCLKONLY_HPH_THSD__POR,
	[TOMTOM_A_CDC_CLSH_FCLKONLY_EAR_THSD] =
				TOMTOM_A_CDC_CLSH_FCLKONLY_EAR_THSD__POR,
	[TOMTOM_A_CDC_CLSH_K_ADDR] = TOMTOM_A_CDC_CLSH_K_ADDR__POR,
	[TOMTOM_A_CDC_CLSH_K_DATA] = TOMTOM_A_CDC_CLSH_K_DATA__POR,
	[TOMTOM_A_CDC_CLSH_I_PA_FACT_HPH_L] =
				TOMTOM_A_CDC_CLSH_I_PA_FACT_HPH_L__POR,
	[TOMTOM_A_CDC_CLSH_I_PA_FACT_HPH_U] =
				TOMTOM_A_CDC_CLSH_I_PA_FACT_HPH_U__POR,
	[TOMTOM_A_CDC_CLSH_I_PA_FACT_EAR_L] =
				TOMTOM_A_CDC_CLSH_I_PA_FACT_EAR_L__POR,
	[TOMTOM_A_CDC_CLSH_I_PA_FACT_EAR_U] =
				TOMTOM_A_CDC_CLSH_I_PA_FACT_EAR_U__POR,
	[TOMTOM_A_CDC_CLSH_V_PA_HD_EAR] = TOMTOM_A_CDC_CLSH_V_PA_HD_EAR__POR,
	[TOMTOM_A_CDC_CLSH_V_PA_HD_HPH] = TOMTOM_A_CDC_CLSH_V_PA_HD_HPH__POR,
	[TOMTOM_A_CDC_CLSH_V_PA_MIN_EAR] = TOMTOM_A_CDC_CLSH_V_PA_MIN_EAR__POR,
	[TOMTOM_A_CDC_CLSH_V_PA_MIN_HPH] = TOMTOM_A_CDC_CLSH_V_PA_MIN_HPH__POR,
	[TOMTOM_A_CDC_IIR1_GAIN_B1_CTL] = TOMTOM_A_CDC_IIR1_GAIN_B1_CTL__POR,
	[TOMTOM_A_CDC_IIR2_GAIN_B1_CTL] = TOMTOM_A_CDC_IIR2_GAIN_B1_CTL__POR,
	[TOMTOM_A_CDC_IIR1_GAIN_B2_CTL] = TOMTOM_A_CDC_IIR1_GAIN_B2_CTL__POR,
	[TOMTOM_A_CDC_IIR2_GAIN_B2_CTL] = TOMTOM_A_CDC_IIR2_GAIN_B2_CTL__POR,
	[TOMTOM_A_CDC_IIR1_GAIN_B3_CTL] = TOMTOM_A_CDC_IIR1_GAIN_B3_CTL__POR,
	[TOMTOM_A_CDC_IIR2_GAIN_B3_CTL] = TOMTOM_A_CDC_IIR2_GAIN_B3_CTL__POR,
	[TOMTOM_A_CDC_IIR1_GAIN_B4_CTL] = TOMTOM_A_CDC_IIR1_GAIN_B4_CTL__POR,
	[TOMTOM_A_CDC_IIR2_GAIN_B4_CTL] = TOMTOM_A_CDC_IIR2_GAIN_B4_CTL__POR,
	[TOMTOM_A_CDC_IIR1_GAIN_B5_CTL] = TOMTOM_A_CDC_IIR1_GAIN_B5_CTL__POR,
	[TOMTOM_A_CDC_IIR2_GAIN_B5_CTL] = TOMTOM_A_CDC_IIR2_GAIN_B5_CTL__POR,
	[TOMTOM_A_CDC_IIR1_GAIN_B6_CTL] = TOMTOM_A_CDC_IIR1_GAIN_B6_CTL__POR,
	[TOMTOM_A_CDC_IIR2_GAIN_B6_CTL] = TOMTOM_A_CDC_IIR2_GAIN_B6_CTL__POR,
	[TOMTOM_A_CDC_IIR1_GAIN_B7_CTL] = TOMTOM_A_CDC_IIR1_GAIN_B7_CTL__POR,
	[TOMTOM_A_CDC_IIR2_GAIN_B7_CTL] = TOMTOM_A_CDC_IIR2_GAIN_B7_CTL__POR,
	[TOMTOM_A_CDC_IIR1_GAIN_B8_CTL] = TOMTOM_A_CDC_IIR1_GAIN_B8_CTL__POR,
	[TOMTOM_A_CDC_IIR2_GAIN_B8_CTL] = TOMTOM_A_CDC_IIR2_GAIN_B8_CTL__POR,
	[TOMTOM_A_CDC_IIR1_CTL] = TOMTOM_A_CDC_IIR1_CTL__POR,
	[TOMTOM_A_CDC_IIR2_CTL] = TOMTOM_A_CDC_IIR2_CTL__POR,
	[TOMTOM_A_CDC_IIR1_GAIN_TIMER_CTL] =
				TOMTOM_A_CDC_IIR1_GAIN_TIMER_CTL__POR,
	[TOMTOM_A_CDC_IIR2_GAIN_TIMER_CTL] =
				TOMTOM_A_CDC_IIR2_GAIN_TIMER_CTL__POR,
	[TOMTOM_A_CDC_IIR1_COEF_B1_CTL] = TOMTOM_A_CDC_IIR1_COEF_B1_CTL__POR,
	[TOMTOM_A_CDC_IIR2_COEF_B1_CTL] = TOMTOM_A_CDC_IIR2_COEF_B1_CTL__POR,
	[TOMTOM_A_CDC_IIR1_COEF_B2_CTL] = TOMTOM_A_CDC_IIR1_COEF_B2_CTL__POR,
	[TOMTOM_A_CDC_IIR2_COEF_B2_CTL] = TOMTOM_A_CDC_IIR2_COEF_B2_CTL__POR,
	[TOMTOM_A_CDC_TOP_GAIN_UPDATE] = TOMTOM_A_CDC_TOP_GAIN_UPDATE__POR,
	[TOMTOM_A_CDC_PA_RAMP_B1_CTL] = TOMTOM_A_CDC_PA_RAMP_B1_CTL__POR,
	[TOMTOM_A_CDC_PA_RAMP_B2_CTL] = TOMTOM_A_CDC_PA_RAMP_B2_CTL__POR,
	[TOMTOM_A_CDC_PA_RAMP_B3_CTL] = TOMTOM_A_CDC_PA_RAMP_B3_CTL__POR,
	[TOMTOM_A_CDC_PA_RAMP_B4_CTL] = TOMTOM_A_CDC_PA_RAMP_B4_CTL__POR,
	[TOMTOM_A_CDC_SPKR_CLIPDET_B1_CTL] =
				TOMTOM_A_CDC_SPKR_CLIPDET_B1_CTL__POR,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_B1_CTL] =
				TOMTOM_A_CDC_SPKR2_CLIPDET_B1_CTL__POR,
	[TOMTOM_A_CDC_COMP0_B1_CTL] = TOMTOM_A_CDC_COMP0_B1_CTL__POR,
	[TOMTOM_A_CDC_COMP1_B1_CTL] = TOMTOM_A_CDC_COMP1_B1_CTL__POR,
	[TOMTOM_A_CDC_COMP2_B1_CTL] = TOMTOM_A_CDC_COMP2_B1_CTL__POR,
	[TOMTOM_A_CDC_COMP0_B2_CTL] = TOMTOM_A_CDC_COMP0_B2_CTL__POR,
	[TOMTOM_A_CDC_COMP1_B2_CTL] = TOMTOM_A_CDC_COMP1_B2_CTL__POR,
	[TOMTOM_A_CDC_COMP2_B2_CTL] = TOMTOM_A_CDC_COMP2_B2_CTL__POR,
	[TOMTOM_A_CDC_COMP0_B3_CTL] = TOMTOM_A_CDC_COMP0_B3_CTL__POR,
	[TOMTOM_A_CDC_COMP1_B3_CTL] = TOMTOM_A_CDC_COMP1_B3_CTL__POR,
	[TOMTOM_A_CDC_COMP2_B3_CTL] = TOMTOM_A_CDC_COMP2_B3_CTL__POR,
	[TOMTOM_A_CDC_COMP0_B4_CTL] = TOMTOM_A_CDC_COMP0_B4_CTL__POR,
	[TOMTOM_A_CDC_COMP1_B4_CTL] = TOMTOM_A_CDC_COMP1_B4_CTL__POR,
	[TOMTOM_A_CDC_COMP2_B4_CTL] = TOMTOM_A_CDC_COMP2_B4_CTL__POR,
	[TOMTOM_A_CDC_COMP0_B5_CTL] = TOMTOM_A_CDC_COMP0_B5_CTL__POR,
	[TOMTOM_A_CDC_COMP1_B5_CTL] = TOMTOM_A_CDC_COMP1_B5_CTL__POR,
	[TOMTOM_A_CDC_COMP2_B5_CTL] = TOMTOM_A_CDC_COMP2_B5_CTL__POR,
	[TOMTOM_A_CDC_COMP0_B6_CTL] = TOMTOM_A_CDC_COMP0_B6_CTL__POR,
	[TOMTOM_A_CDC_COMP1_B6_CTL] = TOMTOM_A_CDC_COMP1_B6_CTL__POR,
	[TOMTOM_A_CDC_COMP2_B6_CTL] = TOMTOM_A_CDC_COMP2_B6_CTL__POR,
	[TOMTOM_A_CDC_COMP0_SHUT_DOWN_STATUS] =
				TOMTOM_A_CDC_COMP0_SHUT_DOWN_STATUS__POR,
	[TOMTOM_A_CDC_COMP1_SHUT_DOWN_STATUS] =
				TOMTOM_A_CDC_COMP1_SHUT_DOWN_STATUS__POR,
	[TOMTOM_A_CDC_COMP2_SHUT_DOWN_STATUS] =
				TOMTOM_A_CDC_COMP2_SHUT_DOWN_STATUS__POR,
	[TOMTOM_A_CDC_COMP0_FS_CFG] = TOMTOM_A_CDC_COMP0_FS_CFG__POR,
	[TOMTOM_A_CDC_COMP1_FS_CFG] = TOMTOM_A_CDC_COMP1_FS_CFG__POR,
	[TOMTOM_A_CDC_COMP2_FS_CFG] = TOMTOM_A_CDC_COMP2_FS_CFG__POR,
	[TOMTOM_A_CDC_CONN_RX1_B1_CTL] = TOMTOM_A_CDC_CONN_RX1_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX1_B2_CTL] = TOMTOM_A_CDC_CONN_RX1_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX1_B3_CTL] = TOMTOM_A_CDC_CONN_RX1_B3_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX2_B1_CTL] = TOMTOM_A_CDC_CONN_RX2_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX2_B2_CTL] = TOMTOM_A_CDC_CONN_RX2_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX2_B3_CTL] = TOMTOM_A_CDC_CONN_RX2_B3_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX3_B1_CTL] = TOMTOM_A_CDC_CONN_RX3_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX3_B2_CTL] = TOMTOM_A_CDC_CONN_RX3_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX4_B1_CTL] = TOMTOM_A_CDC_CONN_RX4_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX4_B2_CTL] = TOMTOM_A_CDC_CONN_RX4_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX5_B1_CTL] = TOMTOM_A_CDC_CONN_RX5_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX5_B2_CTL] = TOMTOM_A_CDC_CONN_RX5_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX6_B1_CTL] = TOMTOM_A_CDC_CONN_RX6_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX6_B2_CTL] = TOMTOM_A_CDC_CONN_RX6_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX7_B1_CTL] = TOMTOM_A_CDC_CONN_RX7_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX7_B2_CTL] = TOMTOM_A_CDC_CONN_RX7_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX7_B3_CTL] = TOMTOM_A_CDC_CONN_RX7_B3_CTL__POR,
	[TOMTOM_A_CDC_CONN_ANC_B1_CTL] = TOMTOM_A_CDC_CONN_ANC_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_ANC_B2_CTL] = TOMTOM_A_CDC_CONN_ANC_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_B1_CTL] = TOMTOM_A_CDC_CONN_TX_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_B2_CTL] = TOMTOM_A_CDC_CONN_TX_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_B3_CTL] = TOMTOM_A_CDC_CONN_TX_B3_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_B4_CTL] = TOMTOM_A_CDC_CONN_TX_B4_CTL__POR,
	[TOMTOM_A_CDC_CONN_EQ1_B1_CTL] = TOMTOM_A_CDC_CONN_EQ1_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_EQ1_B2_CTL] = TOMTOM_A_CDC_CONN_EQ1_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_EQ1_B3_CTL] = TOMTOM_A_CDC_CONN_EQ1_B3_CTL__POR,
	[TOMTOM_A_CDC_CONN_EQ1_B4_CTL] = TOMTOM_A_CDC_CONN_EQ1_B4_CTL__POR,
	[TOMTOM_A_CDC_CONN_EQ2_B1_CTL] = TOMTOM_A_CDC_CONN_EQ2_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_EQ2_B2_CTL] = TOMTOM_A_CDC_CONN_EQ2_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_EQ2_B3_CTL] = TOMTOM_A_CDC_CONN_EQ2_B3_CTL__POR,
	[TOMTOM_A_CDC_CONN_EQ2_B4_CTL] = TOMTOM_A_CDC_CONN_EQ2_B4_CTL__POR,
	[TOMTOM_A_CDC_CONN_SRC1_B1_CTL] = TOMTOM_A_CDC_CONN_SRC1_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_SRC1_B2_CTL] = TOMTOM_A_CDC_CONN_SRC1_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_SRC2_B1_CTL] = TOMTOM_A_CDC_CONN_SRC2_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_SRC2_B2_CTL] = TOMTOM_A_CDC_CONN_SRC2_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_SB_B1_CTL] = TOMTOM_A_CDC_CONN_TX_SB_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_SB_B2_CTL] = TOMTOM_A_CDC_CONN_TX_SB_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_SB_B3_CTL] = TOMTOM_A_CDC_CONN_TX_SB_B3_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_SB_B4_CTL] = TOMTOM_A_CDC_CONN_TX_SB_B4_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_SB_B5_CTL] = TOMTOM_A_CDC_CONN_TX_SB_B5_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_SB_B6_CTL] = TOMTOM_A_CDC_CONN_TX_SB_B6_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_SB_B7_CTL] = TOMTOM_A_CDC_CONN_TX_SB_B7_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_SB_B8_CTL] = TOMTOM_A_CDC_CONN_TX_SB_B8_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_SB_B9_CTL] = TOMTOM_A_CDC_CONN_TX_SB_B9_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_SB_B10_CTL] =
				TOMTOM_A_CDC_CONN_TX_SB_B10_CTL__POR,
	[TOMTOM_A_CDC_CONN_TX_SB_B11_CTL] =
				TOMTOM_A_CDC_CONN_TX_SB_B11_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX_SB_B1_CTL] = TOMTOM_A_CDC_CONN_RX_SB_B1_CTL__POR,
	[TOMTOM_A_CDC_CONN_RX_SB_B2_CTL] = TOMTOM_A_CDC_CONN_RX_SB_B2_CTL__POR,
	[TOMTOM_A_CDC_CONN_CLSH_CTL] = TOMTOM_A_CDC_CONN_CLSH_CTL__POR,
	[TOMTOM_A_CDC_CONN_MISC] = TOMTOM_A_CDC_CONN_MISC__POR,
	[TOMTOM_A_CDC_CONN_RX8_B1_CTL] = TOMTOM_A_CDC_CONN_RX8_B1_CTL__POR,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR_B1_CTL] =
				TOMTOM_A_CDC_CLIP_ADJ_SPKR_B1_CTL__POR,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR_CLIP_LEVEL_ADJUST] =
			TOMTOM_A_CDC_CLIP_ADJ_SPKR_CLIP_LEVEL_ADJUST__POR,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR_MIN_CLIP_THRESHOLD] =
			TOMTOM_A_CDC_CLIP_ADJ_SPKR_MIN_CLIP_THRESHOLD__POR,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR_THRESHOLD_STATUS] =
			TOMTOM_A_CDC_CLIP_ADJ_SPKR_THRESHOLD_STATUS__POR,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR_SAMPLE_MARK] =
			TOMTOM_A_CDC_CLIP_ADJ_SPKR_SAMPLE_MARK__POR,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR_BOOST_GATING] =
			TOMTOM_A_CDC_CLIP_ADJ_SPKR_BOOST_GATING__POR,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR2_B1_CTL] =
				TOMTOM_A_CDC_CLIP_ADJ_SPKR2_B1_CTL__POR,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR2_CLIP_LEVEL_ADJUST] =
			TOMTOM_A_CDC_CLIP_ADJ_SPKR2_CLIP_LEVEL_ADJUST__POR,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR2_MIN_CLIP_THRESHOLD] =
			TOMTOM_A_CDC_CLIP_ADJ_SPKR2_MIN_CLIP_THRESHOLD__POR,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR2_THRESHOLD_STATUS] =
			TOMTOM_A_CDC_CLIP_ADJ_SPKR2_THRESHOLD_STATUS__POR,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR2_SAMPLE_MARK] =
				TOMTOM_A_CDC_CLIP_ADJ_SPKR2_SAMPLE_MARK__POR,
	[TOMTOM_A_CDC_CLIP_ADJ_SPKR2_BOOST_GATING] =
				TOMTOM_A_CDC_CLIP_ADJ_SPKR2_BOOST_GATING__POR,
	[TOMTOM_A_CDC_MBHC_EN_CTL] = TOMTOM_A_CDC_MBHC_EN_CTL__POR,
	[TOMTOM_A_CDC_MBHC_FIR_B1_CFG] = TOMTOM_A_CDC_MBHC_FIR_B1_CFG__POR,
	[TOMTOM_A_CDC_MBHC_FIR_B2_CFG] = TOMTOM_A_CDC_MBHC_FIR_B2_CFG__POR,
	[TOMTOM_A_CDC_MBHC_TIMER_B1_CTL] = TOMTOM_A_CDC_MBHC_TIMER_B1_CTL__POR,
	[TOMTOM_A_CDC_MBHC_TIMER_B2_CTL] = TOMTOM_A_CDC_MBHC_TIMER_B2_CTL__POR,
	[TOMTOM_A_CDC_MBHC_TIMER_B3_CTL] = TOMTOM_A_CDC_MBHC_TIMER_B3_CTL__POR,
	[TOMTOM_A_CDC_MBHC_TIMER_B4_CTL] = TOMTOM_A_CDC_MBHC_TIMER_B4_CTL__POR,
	[TOMTOM_A_CDC_MBHC_TIMER_B5_CTL] = TOMTOM_A_CDC_MBHC_TIMER_B5_CTL__POR,
	[TOMTOM_A_CDC_MBHC_TIMER_B6_CTL] = TOMTOM_A_CDC_MBHC_TIMER_B6_CTL__POR,
	[TOMTOM_A_CDC_MBHC_B1_STATUS] = TOMTOM_A_CDC_MBHC_B1_STATUS__POR,
	[TOMTOM_A_CDC_MBHC_B2_STATUS] = TOMTOM_A_CDC_MBHC_B2_STATUS__POR,
	[TOMTOM_A_CDC_MBHC_B3_STATUS] = TOMTOM_A_CDC_MBHC_B3_STATUS__POR,
	[TOMTOM_A_CDC_MBHC_B4_STATUS] = TOMTOM_A_CDC_MBHC_B4_STATUS__POR,
	[TOMTOM_A_CDC_MBHC_B5_STATUS] = TOMTOM_A_CDC_MBHC_B5_STATUS__POR,
	[TOMTOM_A_CDC_MBHC_B1_CTL] = TOMTOM_A_CDC_MBHC_B1_CTL__POR,
	[TOMTOM_A_CDC_MBHC_B2_CTL] = TOMTOM_A_CDC_MBHC_B2_CTL__POR,
	[TOMTOM_A_CDC_MBHC_VOLT_B1_CTL] = TOMTOM_A_CDC_MBHC_VOLT_B1_CTL__POR,
	[TOMTOM_A_CDC_MBHC_VOLT_B2_CTL] = TOMTOM_A_CDC_MBHC_VOLT_B2_CTL__POR,
	[TOMTOM_A_CDC_MBHC_VOLT_B3_CTL] = TOMTOM_A_CDC_MBHC_VOLT_B3_CTL__POR,
	[TOMTOM_A_CDC_MBHC_VOLT_B4_CTL] = TOMTOM_A_CDC_MBHC_VOLT_B4_CTL__POR,
	[TOMTOM_A_CDC_MBHC_VOLT_B5_CTL] = TOMTOM_A_CDC_MBHC_VOLT_B5_CTL__POR,
	[TOMTOM_A_CDC_MBHC_VOLT_B6_CTL] = TOMTOM_A_CDC_MBHC_VOLT_B6_CTL__POR,
	[TOMTOM_A_CDC_MBHC_VOLT_B7_CTL] = TOMTOM_A_CDC_MBHC_VOLT_B7_CTL__POR,
	[TOMTOM_A_CDC_MBHC_VOLT_B8_CTL] = TOMTOM_A_CDC_MBHC_VOLT_B8_CTL__POR,
	[TOMTOM_A_CDC_MBHC_VOLT_B9_CTL] = TOMTOM_A_CDC_MBHC_VOLT_B9_CTL__POR,
	[TOMTOM_A_CDC_MBHC_VOLT_B10_CTL] = TOMTOM_A_CDC_MBHC_VOLT_B10_CTL__POR,
	[TOMTOM_A_CDC_MBHC_VOLT_B11_CTL] = TOMTOM_A_CDC_MBHC_VOLT_B11_CTL__POR,
	[TOMTOM_A_CDC_MBHC_VOLT_B12_CTL] = TOMTOM_A_CDC_MBHC_VOLT_B12_CTL__POR,
	[TOMTOM_A_CDC_MBHC_CLK_CTL] = TOMTOM_A_CDC_MBHC_CLK_CTL__POR,
	[TOMTOM_A_CDC_MBHC_INT_CTL] = TOMTOM_A_CDC_MBHC_INT_CTL__POR,
	[TOMTOM_A_CDC_MBHC_DEBUG_CTL] = TOMTOM_A_CDC_MBHC_DEBUG_CTL__POR,
	[TOMTOM_A_CDC_MBHC_SPARE] = TOMTOM_A_CDC_MBHC_SPARE__POR,
	[TOMTOM_A_CDC_RX8_B1_CTL] = TOMTOM_A_CDC_RX8_B1_CTL__POR,
	[TOMTOM_A_CDC_RX8_B2_CTL] = TOMTOM_A_CDC_RX8_B2_CTL__POR,
	[TOMTOM_A_CDC_RX8_B3_CTL] = TOMTOM_A_CDC_RX8_B3_CTL__POR,
	[TOMTOM_A_CDC_RX8_B4_CTL] = TOMTOM_A_CDC_RX8_B4_CTL__POR,
	[TOMTOM_A_CDC_RX8_B5_CTL] = TOMTOM_A_CDC_RX8_B5_CTL__POR,
	[TOMTOM_A_CDC_RX8_B6_CTL] = TOMTOM_A_CDC_RX8_B6_CTL__POR,
	[TOMTOM_A_CDC_RX8_VOL_CTL_B1_CTL] =
				TOMTOM_A_CDC_RX8_VOL_CTL_B1_CTL__POR,
	[TOMTOM_A_CDC_RX8_VOL_CTL_B2_CTL] =
				TOMTOM_A_CDC_RX8_VOL_CTL_B2_CTL__POR,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL0] =
				TOMTOM_A_CDC_SPKR2_CLIPDET_VAL0__POR,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL1] =
				TOMTOM_A_CDC_SPKR2_CLIPDET_VAL1__POR,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL2] =
				TOMTOM_A_CDC_SPKR2_CLIPDET_VAL2__POR,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL3] =
				TOMTOM_A_CDC_SPKR2_CLIPDET_VAL3__POR,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL4] =
				TOMTOM_A_CDC_SPKR2_CLIPDET_VAL4__POR,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL5] =
				TOMTOM_A_CDC_SPKR2_CLIPDET_VAL5__POR,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL6] =
				TOMTOM_A_CDC_SPKR2_CLIPDET_VAL6__POR,
	[TOMTOM_A_CDC_SPKR2_CLIPDET_VAL7] =
				TOMTOM_A_CDC_SPKR2_CLIPDET_VAL7__POR,
	[TOMTOM_A_CDC_BOOST_MODE_CTL] = TOMTOM_A_CDC_BOOST_MODE_CTL__POR,
	[TOMTOM_A_CDC_BOOST_THRESHOLD] = TOMTOM_A_CDC_BOOST_THRESHOLD__POR,
	[TOMTOM_A_CDC_BOOST_TAP_SEL] = TOMTOM_A_CDC_BOOST_TAP_SEL__POR,
	[TOMTOM_A_CDC_BOOST_HOLD_TIME] = TOMTOM_A_CDC_BOOST_HOLD_TIME__POR,
	[TOMTOM_A_CDC_BOOST_TRGR_EN] = TOMTOM_A_CDC_BOOST_TRGR_EN__POR,
};
